#!/usr/bin/python3
"""Integration tests for v3i translations resolvers validating country, year, and combined formatters."""

import pytest
from load_one_data import dump_diff, dump_diff_text, dump_same_and_not_same, one_dump_test

from ArWikiCats import resolve_label_ar
from ArWikiCats.new_resolvers.translations_resolvers_v3i.resolve_v3i import resolve_year_job_from_countries

test_data_standard = {
    "10th-century clergy from the Holy Roman Empire": "رجال دين من الإمبراطورية الرومانية المقدسة في القرن 10",
    "10th-century historians from the Fatimid Caliphate": "مؤرخون من الدولة الفاطمية في القرن 10",
    "10th-century Jews from al-Andalus": "يهود من الأندلس في القرن 10",
    "10th-century nobility from the Holy Roman Empire": "نبلاء من الإمبراطورية الرومانية المقدسة في القرن 10",
    "10th-century people from Georgia (country)": "أشخاص من جورجيا في القرن 10",
    "10th-century people from the Abbasid Caliphate": "أشخاص من الدولة العباسية في القرن 10",
    "10th-century people from the Holy Roman Empire": "أشخاص من الإمبراطورية الرومانية المقدسة في القرن 10",
    "10th-century people from West Francia": "أشخاص من مملكة الفرنجة الغربيين في القرن 10",
    "10th-century women from the Abbasid Caliphate": "نساء من الدولة العباسية في القرن 10",
    "10th-century women from the Holy Roman Empire": "نساء من الإمبراطورية الرومانية المقدسة في القرن 10",
    "10th-century writers from the Holy Roman Empire": "كتاب من الإمبراطورية الرومانية المقدسة في القرن 10",
    "11th-century clergy from the Holy Roman Empire": "رجال دين من الإمبراطورية الرومانية المقدسة في القرن 11",
    "11th-century historians from the Abbasid Caliphate": "مؤرخون من الدولة العباسية في القرن 11",
    "11th-century historians from the Holy Roman Empire": "مؤرخون من الإمبراطورية الرومانية المقدسة في القرن 11",
    "11th-century Jews from al-Andalus": "يهود من الأندلس في القرن 11",
    "11th-century nobility from the Holy Roman Empire": "نبلاء من الإمبراطورية الرومانية المقدسة في القرن 11",
    "11th-century people from Bohemia": "أشخاص من بوهيميا في القرن 11",
    "11th-century people from Georgia (country)": "أشخاص من جورجيا في القرن 11",
    "11th-century people from Kievan Rus'": "أشخاص من كييف روس في القرن 11",
    "11th-century people from the Holy Roman Empire": "أشخاص من الإمبراطورية الرومانية المقدسة في القرن 11",
    "11th-century women from Georgia (country)": "نساء من جورجيا في القرن 11",
    "11th-century women from Kievan Rus'": "نساء من كييف روس في القرن 11",
    "11th-century women from the Abbasid Caliphate": "نساء من الدولة العباسية في القرن 11",
    "11th-century women from the Holy Roman Empire": "نساء من الإمبراطورية الرومانية المقدسة في القرن 11",
    "11th-century writers from the Holy Roman Empire": "كتاب من الإمبراطورية الرومانية المقدسة في القرن 11",
    "12th-century deaths from tuberculosis": "وفيات بسبب السل في القرن 12",
    "12th-century historians from the Holy Roman Empire": "مؤرخون من الإمبراطورية الرومانية المقدسة في القرن 12",
    "12th-century Jews from al-Andalus": "يهود من الأندلس في القرن 12",
    "12th-century Jews from the Abbasid Caliphate": "يهود من الدولة العباسية في القرن 12",
    "12th-century nobility from the Holy Roman Empire": "نبلاء من الإمبراطورية الرومانية المقدسة في القرن 12",
    "12th-century people from Bohemia": "أشخاص من بوهيميا في القرن 12",
    "12th-century people from Georgia (country)": "أشخاص من جورجيا في القرن 12",
    "12th-century people from Kievan Rus'": "أشخاص من كييف روس في القرن 12",
    "12th-century people from the Crown of Aragon": "أشخاص من تاج أرغون في القرن 12",
    "12th-century people from the Holy Roman Empire": "أشخاص من الإمبراطورية الرومانية المقدسة في القرن 12",
    "12th-century poets from Georgia (country)": "شعراء من جورجيا في القرن 12",
    "12th-century poets from the Holy Roman Empire": "شعراء من الإمبراطورية الرومانية المقدسة في القرن 12",
    "12th-century women from Georgia (country)": "نساء من جورجيا في القرن 12",
    "12th-century women from Kievan Rus'": "نساء من كييف روس في القرن 12",
    "12th-century women from the Abbasid Caliphate": "نساء من الدولة العباسية في القرن 12",
    "12th-century women from the Holy Roman Empire": "نساء من الإمبراطورية الرومانية المقدسة في القرن 12",
    "12th-century writers from the Holy Roman Empire": "كتاب من الإمبراطورية الرومانية المقدسة في القرن 12",
    "13th-century nobility from the Holy Roman Empire": "نبلاء من الإمبراطورية الرومانية المقدسة في القرن 13",
    "13th-century people from Georgia (country)": "أشخاص من جورجيا في القرن 13",
    "13th-century people from the Crown of Aragon": "أشخاص من تاج أرغون في القرن 13",
    "13th-century people from the Holy Roman Empire": "أشخاص من الإمبراطورية الرومانية المقدسة في القرن 13",
    "13th-century poets from Georgia (country)": "شعراء من جورجيا في القرن 13",
    "13th-century poets from the Holy Roman Empire": "شعراء من الإمبراطورية الرومانية المقدسة في القرن 13",
    "13th-century scientists from the Holy Roman Empire": "علماء من الإمبراطورية الرومانية المقدسة في القرن 13",
    "13th-century women from Bohemia": "نساء من بوهيميا في القرن 13",
    "13th-century women from Georgia (country)": "نساء من جورجيا في القرن 13",
    "13th-century women from the Abbasid Caliphate": "نساء من الدولة العباسية في القرن 13",
    "13th-century women from the Holy Roman Empire": "نساء من الإمبراطورية الرومانية المقدسة في القرن 13",
    "13th-century women from the Ottoman Empire": "نساء من الدولة العثمانية في القرن 13",
    "13th-century writers from the Holy Roman Empire": "كتاب من الإمبراطورية الرومانية المقدسة في القرن 13",
    "14th-century artists from the Holy Roman Empire": "فنانون من الإمبراطورية الرومانية المقدسة في القرن 14",
    "14th-century deaths from tuberculosis": "وفيات بسبب السل في القرن 14",
    "14th-century historians from the Holy Roman Empire": "مؤرخون من الإمبراطورية الرومانية المقدسة في القرن 14",
    "14th-century nobility from the Holy Roman Empire": "نبلاء من الإمبراطورية الرومانية المقدسة في القرن 14",
    "14th-century painters from the Holy Roman Empire": "رسامون من الإمبراطورية الرومانية المقدسة في القرن 14",
    "14th-century people from the Crown of Aragon": "أشخاص من تاج أرغون في القرن 14",
    "14th-century people from the Holy Roman Empire": "أشخاص من الإمبراطورية الرومانية المقدسة في القرن 14",
    "14th-century poets from the Holy Roman Empire": "شعراء من الإمبراطورية الرومانية المقدسة في القرن 14",
    "14th-century scientists from the Holy Roman Empire": "علماء من الإمبراطورية الرومانية المقدسة في القرن 14",
    "14th-century women from Bohemia": "نساء من بوهيميا في القرن 14",
    "14th-century women from Georgia (country)": "نساء من جورجيا في القرن 14",
    "14th-century women from the Holy Roman Empire": "نساء من الإمبراطورية الرومانية المقدسة في القرن 14",
    "14th-century women from the Ottoman Empire": "نساء من الدولة العثمانية في القرن 14",
    "14th-century writers from the Holy Roman Empire": "كتاب من الإمبراطورية الرومانية المقدسة في القرن 14",
    "15th-century artists from the Holy Roman Empire": "فنانون من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century businesspeople from the Holy Roman Empire": "شخصيات أعمال من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century civil servants from the Ottoman Empire": "موظفو خدمة مدنية من الدولة العثمانية في القرن 15",
    "15th-century clergy from the Holy Roman Empire": "رجال دين من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century deaths from tuberculosis": "وفيات بسبب السل في القرن 15",
    "15th-century historians from the Holy Roman Empire": "مؤرخون من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century historians from the Ottoman Empire": "مؤرخون من الدولة العثمانية في القرن 15",
    "15th-century Jews from the Ottoman Empire": "يهود من الدولة العثمانية في القرن 15",
    "15th-century jurists from the Holy Roman Empire": "حقوقيون من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century mathematicians from the Holy Roman Empire": "رياضياتيون من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century nobility from Bohemia": "نبلاء من بوهيميا في القرن 15",
    "15th-century nobility from the Holy Roman Empire": "نبلاء من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century painters from the Holy Roman Empire": "رسامون من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century people from Georgia (country)": "أشخاص من جورجيا في القرن 15",
    "15th-century people from the Holy Roman Empire": "أشخاص من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century physicians from the Holy Roman Empire": "أطباء من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century poets from the Holy Roman Empire": "شعراء من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century scholars from the Holy Roman Empire": "دارسون من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century scientists from the Holy Roman Empire": "علماء من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century scientists from the Ottoman Empire": "علماء من الدولة العثمانية في القرن 15",
    "15th-century sculptors from the Holy Roman Empire": "نحاتون من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century women from Bohemia": "نساء من بوهيميا في القرن 15",
    "15th-century women from the Holy Roman Empire": "نساء من الإمبراطورية الرومانية المقدسة في القرن 15",
    "15th-century women from the Ottoman Empire": "نساء من الدولة العثمانية في القرن 15",
    "15th-century writers from the Holy Roman Empire": "كتاب من الإمبراطورية الرومانية المقدسة في القرن 15",
    "16th-century architects from the Holy Roman Empire": "معماريون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century artists from the Holy Roman Empire": "فنانون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century businesspeople from the Holy Roman Empire": "شخصيات أعمال من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century civil servants from the Ottoman Empire": "موظفو خدمة مدنية من الدولة العثمانية في القرن 16",
    "16th-century clergy from Bohemia": "رجال دين من بوهيميا في القرن 16",
    "16th-century criminals from the Holy Roman Empire": "مجرمون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century deaths from tuberculosis": "وفيات بسبب السل في القرن 16",
    "16th-century educators from the Holy Roman Empire": "معلمون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century engineers from the Holy Roman Empire": "مهندسون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century engravers from the Holy Roman Empire": "نقاشون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century explorers from the Holy Roman Empire": "مستكشفون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century historians from the Holy Roman Empire": "مؤرخون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century historians from the Ottoman Empire": "مؤرخون من الدولة العثمانية في القرن 16",
    "16th-century Jews from the Ottoman Empire": "يهود من الدولة العثمانية في القرن 16",
    "16th-century jurists from the Holy Roman Empire": "حقوقيون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century lawyers from the Holy Roman Empire": "محامون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century mathematicians from the Holy Roman Empire": "رياضياتيون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century musicians from the Holy Roman Empire": "موسيقيون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century nobility from Bohemia": "نبلاء من بوهيميا في القرن 16",
    "16th-century nobility from the Holy Roman Empire": "نبلاء من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century painters from the Holy Roman Empire": "رسامون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century people from New Spain": "أشخاص من إسبانيا الجديدة في القرن 16",
    "16th-century people from Safavid Iran": "أشخاص من إيران الصفوية في القرن 16",
    "16th-century people from the Holy Roman Empire": "أشخاص من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century philosophers from the Holy Roman Empire": "فلاسفة من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century physicians from the Holy Roman Empire": "أطباء من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century poets from the Holy Roman Empire": "شعراء من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century politicians from the Holy Roman Empire": "سياسيون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century rabbis from Bohemia": "حاخامات من بوهيميا في القرن 16",
    "16th-century scholars from the Holy Roman Empire": "دارسون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century scientists from the Holy Roman Empire": "علماء من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century sculptors from the Holy Roman Empire": "نحاتون من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century women from Bohemia": "نساء من بوهيميا في القرن 16",
    "16th-century women from Georgia (country)": "نساء من جورجيا في القرن 16",
    "16th-century women from the Holy Roman Empire": "نساء من الإمبراطورية الرومانية المقدسة في القرن 16",
    "16th-century women from the Ottoman Empire": "نساء من الدولة العثمانية في القرن 16",
    "16th-century writers from Safavid Iran": "كتاب من إيران الصفوية في القرن 16",
    "16th-century writers from the Holy Roman Empire": "كتاب من الإمبراطورية الرومانية المقدسة في القرن 16",
    "17th-century artisans from the Holy Roman Empire": "حرفيون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century artists from the Holy Roman Empire": "فنانون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century biologists from the Holy Roman Empire": "علماء أحياء من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century chemists from the Holy Roman Empire": "كيميائيون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century deaths from tuberculosis": "وفيات بسبب السل في القرن 17",
    "17th-century diplomats from the Holy Roman Empire": "دبلوماسيون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century educators from the Holy Roman Empire": "معلمون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century explorers from the Holy Roman Empire": "مستكشفون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century historians from the Holy Roman Empire": "مؤرخون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century historians from the Ottoman Empire": "مؤرخون من الدولة العثمانية في القرن 17",
    "17th-century Jews from the Ottoman Empire": "يهود من الدولة العثمانية في القرن 17",
    "17th-century jurists from the Holy Roman Empire": "حقوقيون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century lawyers from the Holy Roman Empire": "محامون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century mathematicians from the Holy Roman Empire": "رياضياتيون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century musicians from the Holy Roman Empire": "موسيقيون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century naturalists from the Holy Roman Empire": "علماء طبيعة من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century nobility from Bohemia": "نبلاء من بوهيميا في القرن 17",
    "17th-century nobility from the Holy Roman Empire": "نبلاء من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century painters from the Holy Roman Empire": "رسامون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century people from Connecticut": "أشخاص من كونيتيكت في القرن 17",
    "17th-century people from Massachusetts": "أشخاص من ماساتشوستس في القرن 17",
    "17th-century people from New Spain": "أشخاص من إسبانيا الجديدة في القرن 17",
    "17th-century people from Safavid Iran": "أشخاص من إيران الصفوية في القرن 17",
    "17th-century people from the Dutch Empire": "أشخاص من الإمبراطورية الهولندية في القرن 17",
    "17th-century people from the Holy Roman Empire": "أشخاص من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century people from the Polish–Lithuanian Commonwealth": "أشخاص من الكومنولث البولندي الليتواني في القرن 17",
    "17th-century philosophers from the Holy Roman Empire": "فلاسفة من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century physicians from the Holy Roman Empire": "أطباء من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century poets from Bohemia": "شعراء من بوهيميا في القرن 17",
    "17th-century poets from the Holy Roman Empire": "شعراء من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century politicians from the Holy Roman Empire": "سياسيون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century publishers (people) from the Holy Roman Empire": "ناشرون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century scholars from the Holy Roman Empire": "دارسون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century scholars from the Ottoman Empire": "دارسون من الدولة العثمانية في القرن 17",
    "17th-century scientists from the Holy Roman Empire": "علماء من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century singers from the Holy Roman Empire": "مغنون من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century violinists from the Holy Roman Empire": "عازفو كمان من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century women from Bohemia": "نساء من بوهيميا في القرن 17",
    "17th-century women from Georgia (country)": "نساء من جورجيا في القرن 17",
    "17th-century women from the Holy Roman Empire": "نساء من الإمبراطورية الرومانية المقدسة في القرن 17",
    "17th-century women from the Ottoman Empire": "نساء من الدولة العثمانية في القرن 17",
    "17th-century writers from Safavid Iran": "كتاب من إيران الصفوية في القرن 17",
    "17th-century writers from the Holy Roman Empire": "كتاب من الإمبراطورية الرومانية المقدسة في القرن 17",
    "18th-century actors from the Holy Roman Empire": "ممثلون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century artisans from the Holy Roman Empire": "حرفيون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century artists from Bohemia": "فنانون من بوهيميا في القرن 18",
    "18th-century astronomers from the Russian Empire": "فلكيون من الإمبراطورية الروسية في القرن 18",
    "18th-century biologists from the Holy Roman Empire": "علماء أحياء من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century businesspeople from the Ottoman Empire": "شخصيات أعمال من الدولة العثمانية في القرن 18",
    "18th-century civil servants from the Ottoman Empire": "موظفو خدمة مدنية من الدولة العثمانية في القرن 18",
    "18th-century clergy from Bohemia": "رجال دين من بوهيميا في القرن 18",
    "18th-century clergy from the Russian Empire": "رجال دين من الإمبراطورية الروسية في القرن 18",
    "18th-century criminals from the Holy Roman Empire": "مجرمون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century deaths from tuberculosis": "وفيات بسبب السل في القرن 18",
    "18th-century diplomats from the Holy Roman Empire": "دبلوماسيون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century dramatists and playwrights from the Russian Empire": "كتاب دراما ومسرح من الإمبراطورية الروسية في القرن 18",
    "18th-century educators from the Holy Roman Empire": "معلمون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century educators from the Russian Empire": "معلمون من الإمبراطورية الروسية في القرن 18",
    "18th-century engineers from the Russian Empire": "مهندسون من الإمبراطورية الروسية في القرن 18",
    "18th-century explorers from the Holy Roman Empire": "مستكشفون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century explorers from the Russian Empire": "مستكشفون من الإمبراطورية الروسية في القرن 18",
    "18th-century historians from the Holy Roman Empire": "مؤرخون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century historians from the Ottoman Empire": "مؤرخون من الدولة العثمانية في القرن 18",
    "18th-century historians from the Russian Empire": "مؤرخون من الإمبراطورية الروسية في القرن 18",
    "18th-century Jews from the Ottoman Empire": "يهود من الدولة العثمانية في القرن 18",
    "18th-century Jews from the Russian Empire": "يهود من الإمبراطورية الروسية في القرن 18",
    "18th-century jurists from the Holy Roman Empire": "حقوقيون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century lawyers from the Holy Roman Empire": "محامون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century male actors from the Holy Roman Empire": "ممثلون ذكور من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century male musicians from Bohemia": "موسيقيون ذكور من بوهيميا في القرن 18",
    "18th-century male musicians from the Holy Roman Empire": "موسيقيون ذكور من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century male musicians from the Russian Empire": "موسيقيون ذكور من الإمبراطورية الروسية في القرن 18",
    "18th-century male singers from the Holy Roman Empire": "مغنون ذكور من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century male writers from the Russian Empire": "كتاب ذكور من الإمبراطورية الروسية في القرن 18",
    "18th-century mathematicians from the Holy Roman Empire": "رياضياتيون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century musicians from the Holy Roman Empire": "موسيقيون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century naturalists from the Holy Roman Empire": "علماء طبيعة من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century nobility from Bohemia": "نبلاء من بوهيميا في القرن 18",
    "18th-century nobility from the Holy Roman Empire": "نبلاء من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century nobility from the Russian Empire": "نبلاء من الإمبراطورية الروسية في القرن 18",
    "18th-century non-fiction writers from the Russian Empire": "كتاب غير روائيين من الإمبراطورية الروسية في القرن 18",
    "18th-century painters from the Holy Roman Empire": "رسامون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century people from Connecticut": "أشخاص من كونيتيكت في القرن 18",
    "18th-century people from Delaware": "أشخاص من ديلاوير في القرن 18",
    "18th-century people from Maryland": "أشخاص من ماريلند في القرن 18",
    "18th-century people from Massachusetts": "أشخاص من ماساتشوستس في القرن 18",
    "18th-century people from New Jersey": "أشخاص من نيوجيرسي في القرن 18",
    "18th-century people from New Spain": "أشخاص من إسبانيا الجديدة في القرن 18",
    "18th-century people from New York (state)": "أشخاص من ولاية نيويورك في القرن 18",
    "18th-century people from Pennsylvania": "أشخاص من بنسلفانيا في القرن 18",
    "18th-century people from Safavid Iran": "أشخاص من إيران الصفوية في القرن 18",
    "18th-century people from South Carolina": "أشخاص من كارولاينا الجنوبية في القرن 18",
    "18th-century people from the Dutch Empire": "أشخاص من الإمبراطورية الهولندية في القرن 18",
    "18th-century people from the Holy Roman Empire": "أشخاص من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century people from the Polish–Lithuanian Commonwealth": "أشخاص من الكومنولث البولندي الليتواني في القرن 18",
    "18th-century people from the Russian Empire": "أشخاص من الإمبراطورية الروسية في القرن 18",
    "18th-century people from Virginia": "أشخاص من فرجينيا في القرن 18",
    "18th-century philosophers from the Holy Roman Empire": "فلاسفة من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century philosophers from the Russian Empire": "فلاسفة من الإمبراطورية الروسية في القرن 18",
    "18th-century physicians from the Holy Roman Empire": "أطباء من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century poets from Bohemia": "شعراء من بوهيميا في القرن 18",
    "18th-century poets from Georgia (country)": "شعراء من جورجيا في القرن 18",
    "18th-century poets from the Holy Roman Empire": "شعراء من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century politicians from the Holy Roman Empire": "سياسيون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century publishers (people) from the Holy Roman Empire": "ناشرون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century rabbis from the Russian Empire": "حاخامات من الإمبراطورية الروسية في القرن 18",
    "18th-century scientists from the Holy Roman Empire": "علماء من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century sculptors from Bohemia": "نحاتون من بوهيميا في القرن 18",
    "18th-century sculptors from the Holy Roman Empire": "نحاتون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century singers from the Holy Roman Empire": "مغنون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century violinists from the Holy Roman Empire": "عازفو كمان من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century women from Bohemia": "نساء من بوهيميا في القرن 18",
    "18th-century women from the Holy Roman Empire": "نساء من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century women from the Ottoman Empire": "نساء من الدولة العثمانية في القرن 18",
    "18th-century women musicians from Bohemia": "موسيقيات من بوهيميا في القرن 18",
    "18th-century women musicians from the Russian Empire": "موسيقيات من الإمبراطورية الروسية في القرن 18",
    "18th-century women singers from the Holy Roman Empire": "مغنيات من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century women writers from the Russian Empire": "كاتبات من الإمبراطورية الروسية في القرن 18",
    "18th-century writers from Georgia (country)": "كتاب من جورجيا في القرن 18",
    "18th-century writers from Safavid Iran": "كتاب من إيران الصفوية في القرن 18",
    "18th-century writers from the Holy Roman Empire": "كتاب من الإمبراطورية الرومانية المقدسة في القرن 18",
    "18th-century writers from the Russian Empire": "كتاب من الإمبراطورية الروسية في القرن 18",
    "19th-century actors from the Ottoman Empire": "ممثلون من الدولة العثمانية في القرن 19",
    "19th-century actors from the Russian Empire": "ممثلون من الإمبراطورية الروسية في القرن 19",
    "19th-century biographers from the Russian Empire": "كتاب سيرة من الإمبراطورية الروسية في القرن 19",
    "19th-century chess players from the Russian Empire": "لاعبو شطرنج من الإمبراطورية الروسية في القرن 19",
    "19th-century civil servants from the Ottoman Empire": "موظفو خدمة مدنية من الدولة العثمانية في القرن 19",
    "19th-century classical pianists from the Russian Empire": "عازفو بيانو كلاسيكيون من الإمبراطورية الروسية في القرن 19",
    "19th-century clergy from Ottoman Iraq": "رجال دين من العراق العثماني في القرن 19",
    "19th-century clergy from the Ottoman Empire": "رجال دين من الدولة العثمانية في القرن 19",
    "19th-century clergy from the Russian Empire": "رجال دين من الإمبراطورية الروسية في القرن 19",
    "19th-century composers from the Russian Empire": "ملحنون من الإمبراطورية الروسية في القرن 19",
    "19th-century criminals from the Russian Empire": "مجرمون من الإمبراطورية الروسية في القرن 19",
    "19th-century dancers from the Russian Empire": "راقصون من الإمبراطورية الروسية في القرن 19",
    "19th-century deaths from tuberculosis": "وفيات بسبب السل في القرن 19",
    "19th-century diarists from the Russian Empire": "كتاب يوميات من الإمبراطورية الروسية في القرن 19",
    "19th-century dramatists and playwrights from the Russian Empire": "كتاب دراما ومسرح من الإمبراطورية الروسية في القرن 19",
    "19th-century educators from Georgia (country)": "معلمون من جورجيا في القرن 19",
    "19th-century educators from the Ottoman Empire": "معلمون من الدولة العثمانية في القرن 19",
    "19th-century educators from the Russian Empire": "معلمون من الإمبراطورية الروسية في القرن 19",
    "19th-century engineers from the Russian Empire": "مهندسون من الإمبراطورية الروسية في القرن 19",
    "19th-century explorers from the Russian Empire": "مستكشفون من الإمبراطورية الروسية في القرن 19",
    "19th-century historians from Georgia (country)": "مؤرخون من جورجيا في القرن 19",
    "19th-century historians from the Ottoman Empire": "مؤرخون من الدولة العثمانية في القرن 19",
    "19th-century historians from the Russian Empire": "مؤرخون من الإمبراطورية الروسية في القرن 19",
    "19th-century illustrators from the Russian Empire": "رسامون توضيحيون من الإمبراطورية الروسية في القرن 19",
    "19th-century Jews from Ottoman Iraq": "يهود من العراق العثماني في القرن 19",
    "19th-century Jews from the Ottoman Empire": "يهود من الدولة العثمانية في القرن 19",
    "19th-century Jews from the Russian Empire": "يهود من الإمبراطورية الروسية في القرن 19",
    "19th-century journalists from the Ottoman Empire": "صحفيون من الدولة العثمانية في القرن 19",
    "19th-century journalists from the Russian Empire": "صحفيون من الإمبراطورية الروسية في القرن 19",
    "19th-century landowners from the Russian Empire": "حائزو أراضي من الإمبراطورية الروسية في القرن 19",
    "19th-century lawyers from the Russian Empire": "محامون من الإمبراطورية الروسية في القرن 19",
    "19th-century male actors from the Ottoman Empire": "ممثلون ذكور من الدولة العثمانية في القرن 19",
    "19th-century male musicians from the Russian Empire": "موسيقيون ذكور من الإمبراطورية الروسية في القرن 19",
    "19th-century male singers from the Russian Empire": "مغنون ذكور من الإمبراطورية الروسية في القرن 19",
    "19th-century memoirists from the Russian Empire": "كتاب مذكرات من الإمبراطورية الروسية في القرن 19",
    "19th-century musicians from the Ottoman Empire": "موسيقيون من الدولة العثمانية في القرن 19",
    "19th-century nobility from the Russian Empire": "نبلاء من الإمبراطورية الروسية في القرن 19",
    "19th-century people from Alabama": "أشخاص من ألاباما في القرن 19",
    "19th-century people from Arizona": "أشخاص من أريزونا في القرن 19",
    "19th-century people from Arkansas": "أشخاص من أركنساس في القرن 19",
    "19th-century people from California": "أشخاص من كاليفورنيا في القرن 19",
    "19th-century people from Colorado": "أشخاص من كولورادو في القرن 19",
    "19th-century people from Connecticut": "أشخاص من كونيتيكت في القرن 19",
    "19th-century people from Delaware": "أشخاص من ديلاوير في القرن 19",
    "19th-century people from Florida": "أشخاص من فلوريدا في القرن 19",
    "19th-century people from Georgia (country)": "أشخاص من جورجيا في القرن 19",
    "19th-century people from Georgia (U.S. state)": "أشخاص من ولاية جورجيا في القرن 19",
    "19th-century people from Idaho": "أشخاص من أيداهو في القرن 19",
    "19th-century people from Indiana": "أشخاص من إنديانا في القرن 19",
    "19th-century people from Kansas": "أشخاص من كانساس في القرن 19",
    "19th-century people from Kentucky": "أشخاص من كنتاكي في القرن 19",
    "19th-century people from Louisiana": "أشخاص من لويزيانا في القرن 19",
    "19th-century people from Maine": "أشخاص من مين في القرن 19",
    "19th-century people from Maryland": "أشخاص من ماريلند في القرن 19",
    "19th-century people from Massachusetts": "أشخاص من ماساتشوستس في القرن 19",
    "19th-century people from Michigan": "أشخاص من ميشيغان في القرن 19",
    "19th-century people from Minnesota": "أشخاص من منيسوتا في القرن 19",
    "19th-century people from Mississippi": "أشخاص من مسيسيبي في القرن 19",
    "19th-century people from Missouri": "أشخاص من ميزوري في القرن 19",
    "19th-century people from Montana": "أشخاص من مونتانا في القرن 19",
    "19th-century people from Nebraska": "أشخاص من نبراسكا في القرن 19",
    "19th-century people from Nevada": "أشخاص من نيفادا في القرن 19",
    "19th-century people from New Jersey": "أشخاص من نيوجيرسي في القرن 19",
    "19th-century people from New York (state)": "أشخاص من ولاية نيويورك في القرن 19",
    "19th-century people from North Carolina": "أشخاص من كارولاينا الشمالية في القرن 19",
    "19th-century people from Ohio": "أشخاص من أوهايو في القرن 19",
    "19th-century people from Oklahoma": "أشخاص من أوكلاهوما في القرن 19",
    "19th-century people from Pennsylvania": "أشخاص من بنسلفانيا في القرن 19",
    "19th-century people from Rhode Island": "أشخاص من رود آيلاند في القرن 19",
    "19th-century people from South Carolina": "أشخاص من كارولاينا الجنوبية في القرن 19",
    "19th-century people from South Dakota": "أشخاص من داكوتا الجنوبية في القرن 19",
    "19th-century people from Tennessee": "أشخاص من تينيسي في القرن 19",
    "19th-century people from Texas": "أشخاص من تكساس في القرن 19",
    "19th-century people from the Russian Empire": "أشخاص من الإمبراطورية الروسية في القرن 19",
    "19th-century people from Vermont": "أشخاص من فيرمونت في القرن 19",
    "19th-century people from Virginia": "أشخاص من فرجينيا في القرن 19",
    "19th-century people from Washington (state)": "أشخاص من ولاية واشنطن في القرن 19",
    "19th-century people from Washington, D.C.": "أشخاص من واشنطن العاصمة في القرن 19",
    "19th-century people from Wisconsin": "أشخاص من ويسكونسن في القرن 19",
    "19th-century people from Wyoming": "أشخاص من وايومنغ في القرن 19",
    "19th-century photographers from the Ottoman Empire": "مصورون من الدولة العثمانية في القرن 19",
    "19th-century physicists from the Russian Empire": "فيزيائيون من الإمبراطورية الروسية في القرن 19",
    "19th-century rabbis from Ottoman Iraq": "حاخامات من العراق العثماني في القرن 19",
    "19th-century scientists from Georgia (country)": "علماء من جورجيا في القرن 19",
    "19th-century singers from the Russian Empire": "مغنون من الإمبراطورية الروسية في القرن 19",
    "19th-century women educators from Georgia (country)": "معلمات من جورجيا في القرن 19",
    "19th-century women musicians from the Russian Empire": "موسيقيات من الإمبراطورية الروسية في القرن 19",
    "19th-century women painters from the Russian Empire": "رسامات من الإمبراطورية الروسية في القرن 19",
    "19th-century women scientists from Georgia (country)": "عالمات من جورجيا في القرن 19",
    "19th-century writers from Georgia (country)": "كتاب من جورجيا في القرن 19",
    "19th-century writers from the Russian Empire": "كتاب من الإمبراطورية الروسية في القرن 19",
    "20th-century actors from the Ottoman Empire": "ممثلون من الدولة العثمانية في القرن 20",
    "20th-century architects from Georgia (country)": "معماريون من جورجيا في القرن 20",
    "20th-century artists from Northern Ireland": "فنانون من أيرلندا الشمالية في القرن 20",
    "20th-century autobiographers from Northern Ireland": "كتاب سيرة ذاتية من أيرلندا الشمالية في القرن 20",
    "20th-century bass guitarists from Northern Ireland": "عازفو غيتار باس من أيرلندا الشمالية في القرن 20",
    "20th-century biographers from Northern Ireland": "كتاب سيرة من أيرلندا الشمالية في القرن 20",
    "20th-century biologists from Northern Ireland": "علماء أحياء من أيرلندا الشمالية في القرن 20",
    "20th-century chemists from Northern Ireland": "كيميائيون من أيرلندا الشمالية في القرن 20",
    "20th-century clergy from Northern Ireland": "رجال دين من أيرلندا الشمالية في القرن 20",
    "20th-century clergy from the Ottoman Empire": "رجال دين من الدولة العثمانية في القرن 20",
    "20th-century comedians from Northern Ireland": "كوميديون من أيرلندا الشمالية في القرن 20",
    "20th-century composers from Northern Ireland": "ملحنون من أيرلندا الشمالية في القرن 20",
    "20th-century deaths from infectious disease": "وفيات بسبب أمراض معدية في القرن 20",
    "20th-century deaths from tuberculosis": "وفيات بسبب السل في القرن 20",
    "20th-century dramatists and playwrights from Northern Ireland": "كتاب دراما ومسرح من أيرلندا الشمالية في القرن 20",
    "20th-century drummers from Northern Ireland": "طبالون من أيرلندا الشمالية في القرن 20",
    "20th-century educators from Georgia (country)": "معلمون من جورجيا في القرن 20",
    "20th-century educators from Northern Ireland": "معلمون من أيرلندا الشمالية في القرن 20",
    "20th-century folk musicians from Northern Ireland": "موسيقيو فولك من أيرلندا الشمالية في القرن 20",
    "20th-century guitarists from Northern Ireland": "عازفو قيثارة من أيرلندا الشمالية في القرن 20",
    "20th-century historians from Georgia (country)": "مؤرخون من جورجيا في القرن 20",
    "20th-century historians from Northern Ireland": "مؤرخون من أيرلندا الشمالية في القرن 20",
    "20th-century Jews from Georgia (country)": "يهود من جورجيا في القرن 20",
    "20th-century Jews from the Ottoman Empire": "يهود من الدولة العثمانية في القرن 20",
    "20th-century journalists from Northern Ireland": "صحفيون من أيرلندا الشمالية في القرن 20",
    "20th-century journalists from the Ottoman Empire": "صحفيون من الدولة العثمانية في القرن 20",
    "20th-century lawyers from Northern Ireland": "محامون من أيرلندا الشمالية في القرن 20",
    "20th-century male actors from the Ottoman Empire": "ممثلون ذكور من الدولة العثمانية في القرن 20",
    "20th-century male artists from Northern Ireland": "فنانون ذكور من أيرلندا الشمالية في القرن 20",
    "20th-century male composers from Northern Ireland": "ملحنون ذكور من أيرلندا الشمالية في القرن 20",
    "20th-century male musicians from Northern Ireland": "موسيقيون ذكور من أيرلندا الشمالية في القرن 20",
    "20th-century male singers from Northern Ireland": "مغنون ذكور من أيرلندا الشمالية في القرن 20",
    "20th-century male writers from Northern Ireland": "كتاب ذكور من أيرلندا الشمالية في القرن 20",
    "20th-century mathematicians from Northern Ireland": "رياضياتيون من أيرلندا الشمالية في القرن 20",
    "20th-century medical doctors from Northern Ireland": "أطباء من أيرلندا الشمالية في القرن 20",
    "20th-century memoirists from Northern Ireland": "كتاب مذكرات من أيرلندا الشمالية في القرن 20",
    "20th-century musicians from Georgia (country)": "موسيقيون من جورجيا في القرن 20",
    "20th-century musicians from Northern Ireland": "موسيقيون من أيرلندا الشمالية في القرن 20",
    "20th-century non-fiction writers from Northern Ireland": "كتاب غير روائيين من أيرلندا الشمالية في القرن 20",
    "20th-century novelists from Northern Ireland": "روائيون من أيرلندا الشمالية في القرن 20",
    "20th-century painters from Northern Ireland": "رسامون من أيرلندا الشمالية في القرن 20",
    "20th-century people from Alabama": "أشخاص من ألاباما في القرن 20",
    "20th-century people from Alaska": "أشخاص من ألاسكا في القرن 20",
    "20th-century people from Arizona": "أشخاص من أريزونا في القرن 20",
    "20th-century people from California": "أشخاص من كاليفورنيا في القرن 20",
    "20th-century people from Colorado": "أشخاص من كولورادو في القرن 20",
    "20th-century people from Connecticut": "أشخاص من كونيتيكت في القرن 20",
    "20th-century people from Delaware": "أشخاص من ديلاوير في القرن 20",
    "20th-century people from Florida": "أشخاص من فلوريدا في القرن 20",
    "20th-century people from Georgia (U.S. state)": "أشخاص من ولاية جورجيا في القرن 20",
    "20th-century people from Hawaii": "أشخاص من هاواي في القرن 20",
    "20th-century people from Idaho": "أشخاص من أيداهو في القرن 20",
    "20th-century people from Illinois": "أشخاص من إلينوي في القرن 20",
    "20th-century people from Indiana": "أشخاص من إنديانا في القرن 20",
    "20th-century people from Kansas": "أشخاص من كانساس في القرن 20",
    "20th-century people from Louisiana": "أشخاص من لويزيانا في القرن 20",
    "20th-century people from Maine": "أشخاص من مين في القرن 20",
    "20th-century people from Maryland": "أشخاص من ماريلند في القرن 20",
    "20th-century people from Massachusetts": "أشخاص من ماساتشوستس في القرن 20",
    "20th-century people from Minnesota": "أشخاص من منيسوتا في القرن 20",
    "20th-century people from Mississippi": "أشخاص من مسيسيبي في القرن 20",
    "20th-century people from Missouri": "أشخاص من ميزوري في القرن 20",
    "20th-century people from Montana": "أشخاص من مونتانا في القرن 20",
    "20th-century people from Nebraska": "أشخاص من نبراسكا في القرن 20",
    "20th-century people from Nevada": "أشخاص من نيفادا في القرن 20",
    "20th-century people from New Hampshire": "أشخاص من نيوهامشير في القرن 20",
    "20th-century people from New Jersey": "أشخاص من نيوجيرسي في القرن 20",
    "20th-century people from New York (state)": "أشخاص من ولاية نيويورك في القرن 20",
    "20th-century people from North Carolina": "أشخاص من كارولاينا الشمالية في القرن 20",
    "20th-century people from Northern Ireland": "أشخاص من أيرلندا الشمالية في القرن 20",
    "20th-century people from Ohio": "أشخاص من أوهايو في القرن 20",
    "20th-century people from Oklahoma": "أشخاص من أوكلاهوما في القرن 20",
    "20th-century people from Pennsylvania": "أشخاص من بنسلفانيا في القرن 20",
    "20th-century people from Rhode Island": "أشخاص من رود آيلاند في القرن 20",
    "20th-century people from South Carolina": "أشخاص من كارولاينا الجنوبية في القرن 20",
    "20th-century people from South Dakota": "أشخاص من داكوتا الجنوبية في القرن 20",
    "20th-century people from Tennessee": "أشخاص من تينيسي في القرن 20",
    "20th-century people from Texas": "أشخاص من تكساس في القرن 20",
    "20th-century people from Utah": "أشخاص من يوتا في القرن 20",
    "20th-century people from Vermont": "أشخاص من فيرمونت في القرن 20",
    "20th-century people from Virginia": "أشخاص من فرجينيا في القرن 20",
    "20th-century people from Washington (state)": "أشخاص من ولاية واشنطن في القرن 20",
    "20th-century people from Washington, D.C.": "أشخاص من واشنطن العاصمة في القرن 20",
    "20th-century people from West Virginia": "أشخاص من فرجينيا الغربية في القرن 20",
    "20th-century people from Wisconsin": "أشخاص من ويسكونسن في القرن 20",
    "20th-century people from Wyoming": "أشخاص من وايومنغ في القرن 20",
    "20th-century photographers from Northern Ireland": "مصورون من أيرلندا الشمالية في القرن 20",
    "20th-century physicists from Northern Ireland": "فيزيائيون من أيرلندا الشمالية في القرن 20",
    "20th-century poets from Georgia (country)": "شعراء من جورجيا في القرن 20",
    "20th-century poets from Northern Ireland": "شعراء من أيرلندا الشمالية في القرن 20",
    "20th-century politicians from Georgia (country)": "سياسيون من جورجيا في القرن 20",
    "20th-century politicians from Northern Ireland": "سياسيون من أيرلندا الشمالية في القرن 20",
    "20th-century rabbis from the Ottoman Empire": "حاخامات من الدولة العثمانية في القرن 20",
    "20th-century scientists from Georgia (country)": "علماء من جورجيا في القرن 20",
    "20th-century scientists from Northern Ireland": "علماء من أيرلندا الشمالية في القرن 20",
    "20th-century screenwriters from Northern Ireland": "كتاب سيناريو من أيرلندا الشمالية في القرن 20",
    "20th-century short story writers from Northern Ireland": "كتاب قصة قصيرة من أيرلندا الشمالية في القرن 20",
    "20th-century singer-songwriters from Northern Ireland": "مغنون وكتاب أغاني من أيرلندا الشمالية في القرن 20",
    "20th-century singers from Georgia (country)": "مغنون من جورجيا في القرن 20",
    "20th-century singers from Northern Ireland": "مغنون من أيرلندا الشمالية في القرن 20",
    "20th-century songwriters from Northern Ireland": "كتاب أغان من أيرلندا الشمالية في القرن 20",
    "20th-century sportsmen from Northern Ireland": "رياضيون رجال من أيرلندا الشمالية في القرن 20",
    "20th-century sportspeople from Northern Ireland": "رياضيون من أيرلندا الشمالية في القرن 20",
    "20th-century sportswomen from Northern Ireland": "رياضيات من أيرلندا الشمالية في القرن 20",
    "20th-century women artists from Northern Ireland": "فنانات من أيرلندا الشمالية في القرن 20",
    "20th-century women educators from Georgia (country)": "معلمات من جورجيا في القرن 20",
    "20th-century women educators from Northern Ireland": "معلمات من أيرلندا الشمالية في القرن 20",
    "20th-century women journalists from Northern Ireland": "صحفيات من أيرلندا الشمالية في القرن 20",
    "20th-century women lawyers from Northern Ireland": "محاميات من أيرلندا الشمالية في القرن 20",
    "20th-century women medical doctors from Northern Ireland": "طبيبات من أيرلندا الشمالية في القرن 20",
    "20th-century women musicians from Georgia (country)": "موسيقيات من جورجيا في القرن 20",
    "20th-century women musicians from Northern Ireland": "موسيقيات من أيرلندا الشمالية في القرن 20",
    "20th-century women politicians from Georgia (country)": "سياسيات من جورجيا في القرن 20",
    "20th-century women politicians from Northern Ireland": "سياسيات من أيرلندا الشمالية في القرن 20",
    "20th-century women scientists from Georgia (country)": "عالمات من جورجيا في القرن 20",
    "20th-century women scientists from Northern Ireland": "عالمات من أيرلندا الشمالية في القرن 20",
    "20th-century women singers from Georgia (country)": "مغنيات من جورجيا في القرن 20",
    "20th-century women singers from Northern Ireland": "مغنيات من أيرلندا الشمالية في القرن 20",
    "20th-century women writers from Georgia (country)": "كاتبات من جورجيا في القرن 20",
    "20th-century women writers from Northern Ireland": "كاتبات من أيرلندا الشمالية في القرن 20",
    "20th-century writers from Georgia (country)": "كتاب من جورجيا في القرن 20",
    "20th-century writers from Northern Ireland": "كتاب من أيرلندا الشمالية في القرن 20",
    "20th-century zoologists from Northern Ireland": "علماء حيوانات من أيرلندا الشمالية في القرن 20",
    "21st-century actors from Georgia (country)": "ممثلون من جورجيا في القرن 21",
    "21st-century artists from Northern Ireland": "فنانون من أيرلندا الشمالية في القرن 21",
    "21st-century autobiographers from Northern Ireland": "كتاب سيرة ذاتية من أيرلندا الشمالية في القرن 21",
    "21st-century bass guitarists from Northern Ireland": "عازفو غيتار باس من أيرلندا الشمالية في القرن 21",
    "21st-century biographers from Northern Ireland": "كتاب سيرة من أيرلندا الشمالية في القرن 21",
    "21st-century biologists from Northern Ireland": "علماء أحياء من أيرلندا الشمالية في القرن 21",
    "21st-century chemists from Northern Ireland": "كيميائيون من أيرلندا الشمالية في القرن 21",
    "21st-century clergy from Northern Ireland": "رجال دين من أيرلندا الشمالية في القرن 21",
    "21st-century comedians from Northern Ireland": "كوميديون من أيرلندا الشمالية في القرن 21",
    "21st-century composers from Northern Ireland": "ملحنون من أيرلندا الشمالية في القرن 21",
    "21st-century deaths from tuberculosis": "وفيات بسبب السل في القرن 21",
    "21st-century dramatists and playwrights from Northern Ireland": "كتاب دراما ومسرح من أيرلندا الشمالية في القرن 21",
    "21st-century drummers from Northern Ireland": "طبالون من أيرلندا الشمالية في القرن 21",
    "21st-century educators from Northern Ireland": "معلمون من أيرلندا الشمالية في القرن 21",
    "21st-century folk musicians from Northern Ireland": "موسيقيو فولك من أيرلندا الشمالية في القرن 21",
    "21st-century guitarists from Northern Ireland": "عازفو قيثارة من أيرلندا الشمالية في القرن 21",
    "21st-century historians from Georgia (country)": "مؤرخون من جورجيا في القرن 21",
    "21st-century historians from Northern Ireland": "مؤرخون من أيرلندا الشمالية في القرن 21",
    "21st-century Jews from Georgia (country)": "يهود من جورجيا في القرن 21",
    "21st-century journalists from Northern Ireland": "صحفيون من أيرلندا الشمالية في القرن 21",
    "21st-century lawyers from Northern Ireland": "محامون من أيرلندا الشمالية في القرن 21",
    "21st-century male composers from Northern Ireland": "ملحنون ذكور من أيرلندا الشمالية في القرن 21",
    "21st-century male musicians from Northern Ireland": "موسيقيون ذكور من أيرلندا الشمالية في القرن 21",
    "21st-century male singers from Northern Ireland": "مغنون ذكور من أيرلندا الشمالية في القرن 21",
    "21st-century male writers from Northern Ireland": "كتاب ذكور من أيرلندا الشمالية في القرن 21",
    "21st-century mathematicians from Northern Ireland": "رياضياتيون من أيرلندا الشمالية في القرن 21",
    "21st-century memoirists from Northern Ireland": "كتاب مذكرات من أيرلندا الشمالية في القرن 21",
    "21st-century musicians from Northern Ireland": "موسيقيون من أيرلندا الشمالية في القرن 21",
    "21st-century non-fiction writers from Northern Ireland": "كتاب غير روائيين من أيرلندا الشمالية في القرن 21",
    "21st-century novelists from Northern Ireland": "روائيون من أيرلندا الشمالية في القرن 21",
    "21st-century opera singers from Georgia (country)": "مغنو أوبرا من جورجيا في القرن 21",
    "21st-century people from Alabama": "أشخاص من ألاباما في القرن 21",
    "21st-century people from Alaska": "أشخاص من ألاسكا في القرن 21",
    "21st-century people from Arizona": "أشخاص من أريزونا في القرن 21",
    "21st-century people from California": "أشخاص من كاليفورنيا في القرن 21",
    "21st-century people from Connecticut": "أشخاص من كونيتيكت في القرن 21",
    "21st-century people from Delaware": "أشخاص من ديلاوير في القرن 21",
    "21st-century people from Florida": "أشخاص من فلوريدا في القرن 21",
    "21st-century people from Georgia (country)": "أشخاص من جورجيا في القرن 21",
    "21st-century people from Georgia (U.S. state)": "أشخاص من ولاية جورجيا في القرن 21",
    "21st-century people from Idaho": "أشخاص من أيداهو في القرن 21",
    "21st-century people from Illinois": "أشخاص من إلينوي في القرن 21",
    "21st-century people from Indiana": "أشخاص من إنديانا في القرن 21",
    "21st-century people from Louisiana": "أشخاص من لويزيانا في القرن 21",
    "21st-century people from Maine": "أشخاص من مين في القرن 21",
    "21st-century people from Maryland": "أشخاص من ماريلند في القرن 21",
    "21st-century people from Michigan": "أشخاص من ميشيغان في القرن 21",
    "21st-century people from Minnesota": "أشخاص من منيسوتا في القرن 21",
    "21st-century people from New Jersey": "أشخاص من نيوجيرسي في القرن 21",
    "21st-century people from North Carolina": "أشخاص من كارولاينا الشمالية في القرن 21",
    "21st-century people from Northern Ireland": "أشخاص من أيرلندا الشمالية في القرن 21",
    "21st-century people from Oklahoma": "أشخاص من أوكلاهوما في القرن 21",
    "21st-century people from Pennsylvania": "أشخاص من بنسلفانيا في القرن 21",
    "21st-century people from Rhode Island": "أشخاص من رود آيلاند في القرن 21",
    "21st-century people from South Carolina": "أشخاص من كارولاينا الجنوبية في القرن 21",
    "21st-century people from South Dakota": "أشخاص من داكوتا الجنوبية في القرن 21",
    "21st-century people from Tennessee": "أشخاص من تينيسي في القرن 21",
    "21st-century people from Texas": "أشخاص من تكساس في القرن 21",
    "21st-century people from Utah": "أشخاص من يوتا في القرن 21",
    "21st-century people from Washington (state)": "أشخاص من ولاية واشنطن في القرن 21",
    "21st-century people from Washington, D.C.": "أشخاص من واشنطن العاصمة في القرن 21",
    "21st-century people from Wisconsin": "أشخاص من ويسكونسن في القرن 21",
    "21st-century photographers from Northern Ireland": "مصورون من أيرلندا الشمالية في القرن 21",
    "21st-century physicists from Northern Ireland": "فيزيائيون من أيرلندا الشمالية في القرن 21",
    "21st-century poets from Georgia (country)": "شعراء من جورجيا في القرن 21",
    "21st-century poets from Northern Ireland": "شعراء من أيرلندا الشمالية في القرن 21",
    "21st-century politicians from Northern Ireland": "سياسيون من أيرلندا الشمالية في القرن 21",
    "21st-century scientists from Georgia (country)": "علماء من جورجيا في القرن 21",
    "21st-century scientists from Northern Ireland": "علماء من أيرلندا الشمالية في القرن 21",
    "21st-century screenwriters from Northern Ireland": "كتاب سيناريو من أيرلندا الشمالية في القرن 21",
    "21st-century short story writers from Northern Ireland": "كتاب قصة قصيرة من أيرلندا الشمالية في القرن 21",
    "21st-century singer-songwriters from Northern Ireland": "مغنون وكتاب أغاني من أيرلندا الشمالية في القرن 21",
    "21st-century singers from Northern Ireland": "مغنون من أيرلندا الشمالية في القرن 21",
    "21st-century songwriters from Northern Ireland": "كتاب أغان من أيرلندا الشمالية في القرن 21",
    "21st-century sportsmen from Georgia (country)": "رياضيون رجال من جورجيا في القرن 21",
    "21st-century sportsmen from Northern Ireland": "رياضيون رجال من أيرلندا الشمالية في القرن 21",
    "21st-century sportspeople from Georgia (country)": "رياضيون من جورجيا في القرن 21",
    "21st-century sportspeople from Northern Ireland": "رياضيون من أيرلندا الشمالية في القرن 21",
    "21st-century sportswomen from Georgia (country)": "رياضيات من جورجيا في القرن 21",
    "21st-century sportswomen from Northern Ireland": "رياضيات من أيرلندا الشمالية في القرن 21",
    "21st-century women artists from Northern Ireland": "فنانات من أيرلندا الشمالية في القرن 21",
    "21st-century women composers from Northern Ireland": "ملحنات من أيرلندا الشمالية في القرن 21",
    "21st-century women journalists from Northern Ireland": "صحفيات من أيرلندا الشمالية في القرن 21",
    "21st-century women lawyers from Northern Ireland": "محاميات من أيرلندا الشمالية في القرن 21",
    "21st-century women musicians from Georgia (country)": "موسيقيات من جورجيا في القرن 21",
    "21st-century women opera singers from Georgia (country)": "مغنيات أوبرا من جورجيا في القرن 21",
    "21st-century women politicians from Georgia (country)": "سياسيات من جورجيا في القرن 21",
    "21st-century women politicians from Northern Ireland": "سياسيات من أيرلندا الشمالية في القرن 21",
    "21st-century women scientists from Northern Ireland": "عالمات من أيرلندا الشمالية في القرن 21",
    "21st-century women singers from Georgia (country)": "مغنيات من جورجيا في القرن 21",
    "21st-century women writers from Northern Ireland": "كاتبات من أيرلندا الشمالية في القرن 21",
    "21st-century writers from Georgia (country)": "كتاب من جورجيا في القرن 21",
    "21st-century writers from Northern Ireland": "كتاب من أيرلندا الشمالية في القرن 21",
    "21st-century zoologists from Northern Ireland": "علماء حيوانات من أيرلندا الشمالية في القرن 21",
    "7th-century deaths from tuberculosis": "وفيات بسبب السل في القرن 7",
    "8th-century historians from the Abbasid Caliphate": "مؤرخون من الدولة العباسية في القرن 8",
    "8th-century Jews from the Abbasid Caliphate": "يهود من الدولة العباسية في القرن 8",
    "8th-century people from the Abbasid Caliphate": "أشخاص من الدولة العباسية في القرن 8",
    "9th-century historians from al-Andalus": "مؤرخون من الأندلس في القرن 9",
    "9th-century historians from the Abbasid Caliphate": "مؤرخون من الدولة العباسية في القرن 9",
    "9th-century Jews from al-Andalus": "يهود من الأندلس في القرن 9",
    "9th-century Jews from the Abbasid Caliphate": "يهود من الدولة العباسية في القرن 9",
    "9th-century people from East Francia": "أشخاص من مملكة الفرنجة الشرقيين في القرن 9",
    "9th-century people from the Abbasid Caliphate": "أشخاص من الدولة العباسية في القرن 9",
    "9th-century people from West Francia": "أشخاص من مملكة الفرنجة الغربيين في القرن 9",
}


@pytest.mark.parametrize("category,expected", test_data_standard.items(), ids=test_data_standard.keys())
@pytest.mark.slow
def test_resolve_v3i_new_1(category: str, expected: str) -> None:
    """Test resolve year job from countries function for test_data_standard."""
    result1 = resolve_year_job_from_countries(category)
    assert result1 == expected

    result2 = resolve_label_ar(category)
    assert result2 == expected


to_test = [
    ("test_resolve_v3i_new_1", test_data_standard, resolve_year_job_from_countries),
]


@pytest.mark.parametrize("name,data,callback", to_test)
@pytest.mark.dump
def test_dump_all(name: str, data: dict[str, str], callback) -> None:
    expected, diff_result = one_dump_test(data, callback)
    dump_diff(diff_result, name)
    # dump_diff_text(expected, diff_result, name)
    # dump_same_and_not_same(data, diff_result, name, just_dump=True)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



#
import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats import resolve_arabic_category_label
from ArWikiCats.new_resolvers.countries_names_resolvers.us_states import normalize_state
from ArWikiCats.translations import US_STATES

test_data = {
    "Category:{en} in the War of 1812": "تصنيف:{ar} في حرب 1812",
    "Category:{en} Democrats": "تصنيف:ديمقراطيون من ولاية {ar}",
    "Category:{en} lawyers": "تصنيف:محامون من ولاية {ar}",
    "Category:{en} state court judges": "تصنيف:قضاة محكمة ولاية {ar}",
    "Category:{en} state courts": "تصنيف:محكمة ولاية {ar}",
    "Category:{en} state senators": "تصنيف:أعضاء مجلس شيوخ ولاية {ar}",
}


washington_data = {
    "Category:washington, d.c. Democrats": "تصنيف:ديمقراطيون من واشنطن العاصمة",
    "Category:washington, d.c. lawyers": "تصنيف:محامون من واشنطن العاصمة",
    "Category:washington, d.c. state court judges": "تصنيف:قضاة محكمة واشنطن العاصمة",
    "Category:washington, d.c. state courts": "تصنيف:محكمة واشنطن العاصمة",
    "Category:washington, d.c. state senators": "تصنيف:أعضاء مجلس شيوخ واشنطن العاصمة",
}

data_1 = {}
all_test_data = {}

for en, ar in US_STATES.items():
    test_one = {x.format(en=en): normalize_state(v.format(ar=ar)) for x, v in test_data.items()}
    data_1.setdefault(en, test_one)
    all_test_data.update(test_one)

data_1["washington, d.c."] = washington_data
all_test_data.update(washington_data)

to_test = [(f"test_us_counties_{x}", v) for x, v in data_1.items()]

to_test.append(("test_all_test_data", all_test_data))


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_all_dump(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_arabic_category_label)
    if diff_result:
        dump_diff(diff_result, name)
        dump_diff(expected, f"{name}_expected")

    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"


@pytest.mark.parametrize("input_text,expected", all_test_data.items(), ids=all_test_data.keys())
@pytest.mark.slow
def test_all_data(input_text: str, expected: str) -> None:
    result = resolve_arabic_category_label(input_text)
    assert result == expected



#!/usr/bin/python3
"""
Arabic label translation for general categories.

This module provides functionality to translate English category names
into Arabic labels by applying various resolution strategies.
"""

import functools
import re
from typing import Optional

from ...fix import fixtitle
from ...format_bots.relation_mapping import translation_category_relations
from ...helps import dump_data, logger
from ...time_resolvers import time_to_arabic
from ...translations import Jobs_new, jobs_mens_data
from ...utils import get_relation_word, get_value_from_any_table
from ..ma_bots2.ar_lab_bot import find_ar_label
from ..make_bots.bot_2018 import get_pop_All_18
from ..matables_bots.bot import Films_O_TT, players_new_keys

en_literes = "[abcdefghijklmnopqrstuvwxyz]"


@functools.lru_cache(maxsize=10000)
def find_lab(category: str, category_r: str) -> str:
    """Find a label for the given category using multiple data sources.

    Args:
        category: The normalized category string to look up
        category_r: The original category string for logging

    Returns:
        The found Arabic label or empty string if not found
    """
    cate_low = category.lower()

    _lab = Films_O_TT.get(cate_low, "")

    if not _lab:
        _lab = get_pop_All_18(cate_low, "")

    if not _lab:
        _lab = get_value_from_any_table(cate_low, [players_new_keys, jobs_mens_data, Jobs_new])

    if not _lab:
        _lab = time_to_arabic.convert_time_to_arabic(cate_low)

    if _lab:
        logger.info(f'>>>> <<lightyellow>>test: cat "{category_r}", {_lab=}')
        logger.info(f'>>>> <<lightyellow>> cat:"{category_r}", {_lab=}')

    return _lab


@functools.lru_cache(maxsize=10000)
# @dump_data(1, input_keys=["category", "start_get_country2"])
def work_separator_names(
    category: str,
    cate_test: str = "",
    start_get_country2: bool = False,
) -> str:
    """Process categories that contain relational words (separator).

    This function extracts relational words from categories and uses them
    to find appropriate Arabic labels.

    Args:
        category: The category string to process
        cate_test: Optional test category string
        start_get_country2: Whether to start country lookup

    Returns:
        The associated Arabic label if found, otherwise an empty string.
    """
    separator, separator_name = get_relation_word(category, translation_category_relations)

    if not separator:
        return ""

    logger.info(f'<<lightblue>>>>>> work_separator_names: separator:"{separator_name}":"{separator}" in category ')
    arlabel = find_ar_label(category, separator, cate_test=cate_test, start_get_country2=start_get_country2)

    if not arlabel:
        return ""

    # Check if the result contains Arabic characters
    if re.sub(en_literes, "", arlabel, flags=re.IGNORECASE) != arlabel:
        arlabel = ""

    logger.info(f">>>> <<lightyellow>> {arlabel=}")

    return arlabel


def _translate_general_category(category_r: str, category: str, start_get_country2: bool = True) -> str:
    """Translate an English category to Arabic label.

    This function processes a category string by normalizing the format
    and applying various strategies to find an appropriate Arabic label.

    Args:
        category_r: The input category string that needs to be processed
        start_get_country2: A flag indicating whether to retrieve country-related data.
            Defaults to True.

    Returns:
        The processed Arabic label associated with the input category.
    """
    cate_test = category.lower()

    arlabel = get_pop_All_18(category, "")

    if not arlabel:
        arlabel = find_lab(category, category_r)

    if not arlabel:
        arlabel = work_separator_names(category, cate_test, start_get_country2=start_get_country2)

    return arlabel


@functools.lru_cache(maxsize=10000)
def translate_general_category(category_r: str, start_get_country2: bool = True, fix_title: bool = True) -> str:
    """
    "Category:20th-century musicians by instrument from Northern Ireland": "تصنيف:موسيقيون في القرن 20 حسب الآلة من أيرلندا الشمالية",
    "Category:21st-century musicians by instrument from Northern Ireland": "تصنيف:موسيقيون في القرن 21 حسب الآلة من أيرلندا الشمالية",
    """
    category = category_r.replace("_", " ")
    category = re.sub(r"category:", "", category, flags=re.IGNORECASE)

    logger.info(f"<<lightyellow>>>> ^^^^^^^^^ translate_general_category start ^^^^^^^^^ ({category}) ")
    logger.debug(f"<<lightyellow>>>>>> {category_r=}, {start_get_country2=}, {fix_title=}")

    arlabel = _translate_general_category(category_r, category, start_get_country2)

    if arlabel and fix_title:
        arlabel = fixtitle.fixlabel(arlabel, en=category_r)
        logger.info(f'>>>>>> <<green>>test: cat "{category_r}", {arlabel=}')

    if arlabel:
        logger.debug(f"<<lightyellow>>>> translate_general_category {arlabel=}  ")

    logger.debug("<<lightyellow>>>> ^^^^^^^^^ translate_general_category end ^^^^^^^^^ ")

    return arlabel



"""
Tests
"""

import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats.new_resolvers.jobs_resolvers.womens import womens_resolver_labels

test_womens_data_0 = {}

test_womens_data = {
    "women children's writers": "كاتبات أطفال",
    "female women's rights activists": "ناشطات في حقوق المرأة",
    "expatriate women's football players": "لاعبات كرة قدم مغتربات",
    "female alpine skiers": "متزحلقات منحدرات ثلجية",
    "female archers": "نبالات",
    "female artistic gymnasts": "لاعبات جمباز فني",
    "female badminton players": "لاعبات تنس ريشة",
    "female biathletes": "لاعبات بياثلون",
    "female bobsledders": "متزلجات جماعيات",
    "female bodybuilders": "لاعبات كمال أجسام",
    "female bowls players": "لاعبات بولينج",
    "female canoeists": "متسابقات قوارب الكانوي",
    "female climbers": "متسلقات",
    "female comics artists": "فنانات قصص مصورة",
    "female comics writers": "كاتبات قصص مصورة",
    "female criminals": "مجرمات",
    "female cross-country skiers": "متزحلقات ريفيات",
    "female curlers": "لاعبات كيرلنغ",
    "female cyclists": "دراجات",
    "female dancers": "راقصات",
    "female divers": "غواصات",
    "female equestrians": "فارسات خيول",
    "female fencers": "مبارزات",
    "female field hockey players": "لاعبات هوكي ميدان",
    "female figure skaters": "متزلجات فنيات",
    "female footballers": "لاعبات كرة قدم",
    "female freestyle skiers": "ممارسات تزلج حر",
    "female freestyle swimmers": "سباحات تزلج حر",
    "female golfers": "لاعبات غولف",
    "female handball players": "لاعبات كرة يد",
    "female ice dancers": "راقصات جليد",
    "female judoka": "لاعبات جودو",
    "female kabaddi players": "لاعبات كابادي",
    "female karateka": "ممارسات كاراتيه",
    "female kickboxers": "مقاتلات كيك بوكسنغ",
    "female lugers": "زاحفات ثلجيات",
    "female martial artists": "ممارسات فنون قتالية",
    "female mixed martial artists": "مقاتلات فنون قتالية مختلطة",
    "female models": "عارضات أزياء",
    "female modern pentathletes": "متسابقات خماسي حديث",
    "female muay thai practitioners": "ممارسات موياي تاي",
    "female murder victims": "ضحيات قتل",
    "female nurses": "ممرضات",
    "female pair skaters": "متزلجات فنيات على الجليد",
    "female professional wrestlers": "مصارعات محترفات",
    "female racing drivers": "سائقات سيارات سباق",
    "female rowers": "مجدفات",
    "female rugby league players": "لاعبات دوري رجبي",
    "female rugby union players": "لاعبات اتحاد رجبي",
    "female sailors (sport)": "بحارات رياضيات",
    "female single skaters": "متزلجات فرديات",
    "female skateboarders": "متزلجات ألواح",
    "female skeleton racers": "متزلجات صدريات",
    "female ski jumpers": "متزلجات قفز",
    "female skiers": "متزحلقات",
    "female snowboarders": "متزلجات على الثلج",
    "female speed skaters": "متزلجات سرعة",
    "female spies": "جاسوسات",
    "female sport shooters": "لاعبات رماية",
    "female sport wrestlers": "مصارعات رياضيات",
    "female sports coaches": "مدربات رياضة",
    "female sports-people": "رياضيات",
    "female squash players": "لاعبات اسكواش",
    "female swimmers": "سباحات",
    "female table tennis players": "لاعبات كرة طاولة",
    "female taekwondo practitioners": "لاعبات تايكوندو",
    "female tennis players": "لاعبات كرة مضرب",
    "female triathletes": "لاعبات ترياثلون",
    "female water polo players": "لاعبات كرة ماء",
    "female weightlifters": "رباعات",
    "women academics": "أكاديميات",
    "women accordionists": "عازفات أكورديات",
    "women accountants": "محاسبات",
    "women activists": "ناشطات",
    "women animators": "رسامات رسوم متحركة",
    "women anthropologists": "عالمات أنثروبولوجيا",
    "women archaeologists": "عالمات آثار",
    "women architects": "معماريات",
    "women art collectors": "جامعات فنون",
    "women art critics": "ناقدات فنيات",
    "women art dealers": "تاجرات أعمال فنية",
    "women artists": "فنانات",
    "women athletes": "لاعبات قوى",
    "women baseball players": "لاعبات كرة قاعدة",
    "women biographers": "كاتبات سيرة",
    "women bloggers": "مدونات",
    "women botanists": "عالمات نباتات",
    "women boxers": "ملاكمات",
    "women business executives": "مسيرات أعمال",
    "women centenarians": "مئويات",
    "women ceramists": "خزفيات",
    "women chefs": "طباخات",
    "women chemists": "كيميائيات",
    "women choreographers": "مصممات رقص",
    "women cinematographers": "مصورات سينمائيات",
    "women civil servants": "موظفات خدمة مدنية",
    "women classical composers": "ملحنات كلاسيكيات",
    "women classical pianists": "عازفات بيانو كلاسيكيات",
    "women classical violinists": "عازفات كمان كلاسيكيات",
    "women columnists": "كاتبات عمود",
    "women comedians": "كوميديات",
    "women company founders": "مؤسسات شركات",
    "women composers": "ملحنات",
    "women computer scientists": "عالمات حاسوب",
    "women conductors (music)": "قائدات فرق موسيقية",
    "women cricketers": "لاعبات كريكت",
    "women critics": "ناقدات",
    "women curators": "أمينات",
    "women designers": "مصممات",
    "women diarists": "كاتبات يوميات",
    "women diplomats": "دبلوماسيات",
    "women directors": "مخرجات",
    "women dramatists and playwrights": "كاتبات دراما ومسرح",
    "women dramatists": "دراميات",
    "women drummers": "طبالات",
    "women earth scientists": "عالمات الأرض",
    "women economists": "اقتصاديات",
    "women editors": "محررات",
    "women educational theorists": "منظرات تربويات",
    "women educators": "معلمات",
    "women engineers": "مهندسات",
    "women environmentalists": "بيئيات",
    "women essayists": "كاتبات مقالات",
    "women farmers": "فلاحات",
    "women fashion designers": "مصممات أزياء",
    "women film directors": "مخرجات أفلام",
    "women film editors": "محررات أفلام",
    "women film producers": "منتجات أفلام",
    "women flautists": "عازفات فولت",
    "women folk musicians": "موسيقيات فولك",
    "women folk singer-songwriters": "مغنيات وكاتبات أغاني فولك",
    "women folk singers": "مغنيات فولك",
    "women folklorists": "فلكلوريات",
    "women food writers": "كاتبات طعام",
    "women governors": "حاكمات",
    "women graphic designers": "مصممات جرافيك",
    "women guitarists": "عازفات قيثارة",
    "women hammer throwers": "راميات مطرقة",
    "women heavy metal singers": "مغنيات هيفي ميتال",
    "women heptathletes": "متسابقات سباعي",
    "women high jumpers": "متسابقات قفز عالي",
    "women historians": "مؤرخات",
    "women human rights activists": "ناشطات في حقوق الإنسان",
    "women humorists": "فكاهيات",
    "women hurdlers": "لاعبات قفز الحواجز",
    "women illustrators": "رسامات توضيحيات",
    "women inventors": "مخترعات",
    "women jazz composers": "ملحنات جاز",
    "women jazz musicians": "موسيقيات جاز",
    "women jazz singers": "مغنيات جاز",
    "women journalists": "صحفيات",
    "women judges": "قاضيات",
    "women jurists": "حقوقيات",
    "women landowners": "حائزات أراضي",
    "women landscape architects": "مهندسات مناظر طبيعية",
    "women lawyers": "محاميات",
    "women librarians": "أمينات مكتبات",
    "women linguists": "لغويات",
    "women literary critics": "ناقدات أدبيات",
    "women long jumpers": "لاعبات قفز طويل",
    "women long-distance runners": "عداءات مسافات طويلة",
    "women marathon runners": "عداءات ماراثون",
    "women marxists": "ماركسيات",
    "women mathematicians": "عالمات رياضيات",
    "women mayors of places": "رئيسات بلديات",
    "women medical doctors": "طبيبات",
    "women memoirists": "كاتبات مذكرات",
    "women microbiologists": "عالمات أحياء مجهريات",
    "women middle-distance runners": "عداءات مسافات متوسطة",
    "women monarchs": "ملكيات",
    "women motivational speakers": "محفزات",
    "women music educators": "معلمات موسيقى",
    "women musicians": "موسيقيات",
    "women musicologists": "عالمات موسيقى",
    "women neuroscientists": "عالمات أعصاب",
    "women non-fiction writers": "كاتبات غير روائيات",
    "women novelists": "روائيات",
    "women opera singers": "مغنيات أوبرا",
    "women ophthalmologists": "طبيبات عيون",
    "women organists": "عازفات أورغن",
    "women painters": "رسامات",
    "women paleontologists": "عالمات حفريات",
    "women performance artists": "فنانات أدائيات",
    "women philologists": "عالمات لغة",
    "women philosophers": "فلسفيات",
    "women photographers": "مصورات",
    "women physicians": "طبيبات",
    "women physicists": "فيزيائيات",
    "women pianists": "عازفات بيانو",
    "women poets": "شاعرات",
    "women pole vaulters": "قافزات بالزانة",
    "women police officers": "ضابطات شرطة",
    "women politicians": "سياسيات",
    "women pop singers": "مغنيات بوب",
    "women psychiatrists": "طبيبات نفسيات",
    "women radio personalities": "شخصيات راديو",
    "women radio presenters": "مذيعات راديو",
    "women rappers": "مغنيات راب",
    "women referees and umpires": "حكمات رياضات",
    "women rock singers": "مغنيات روك",
    "women rulers": "حاكمات",
    "women runners": "عداءات",
    "women satirists": "كاتبات هجائيات",
    "women saxophonists": "عازفات سكسفون",
    "women scientists": "عالمات",
    "women screenwriters": "كاتبات سيناريو",
    "women sculptors": "نحاتات",
    "women short story writers": "كاتبات قصة قصيرة",
    "women shot putters": "لاعبات دفع ثقل",
    "women singer-songwriters": "مغنيات وكاتبات أغاني",
    "women singers": "مغنيات",
    "women sociologists": "عالمات اجتماع",
    "women songwriters": "كاتبات أغان",
    "women sportswriters": "كاتبات رياضيات",
    "women sprinters": "عداءات سريعات",
    "women state legislators": "مشرعات الولايات",
    "women statisticians": "إحصائيات",
    "women steeplechase runners": "عداءات موانع",
    "women television directors": "مخرجات تلفزيون",
    "women television personalities": "شخصيات تلفزيون",
    "women television presenters": "مذيعات تلفزيون",
    "women television producers": "منتجات تلفزيون",
    "women theatre directors": "مخرجات مسرح",
    "women trade unionists": "نقابيات",
    "women triple jumpers": "لاعبات وثب ثلاثي",
    "women video artists": "فنانات فيديو",
    "women violinists": "عازفات كمان",
    "women wheelchair racers": "متسابقات كراسي متحركة",
    "women writers": "كاتبات",
    "women zoologists": "عالمات حيوانات",
    "women's basketball coaches": "مدربات كرة سلة",
    "women's candidates": "مرشحات",
    "women's football managers": "مدربات كرة قدم",
    "women's football wingers": "جناحات كرة قدم",
    "women's ice hockey defencemen": "مدافعات هوكي جليد",
    "women's ice hockey goaltenders": "حارسات مرمى هوكي جليد",
    "women's rugby union coaches": "مدربات اتحاد رجبي",
}


@pytest.mark.parametrize("category, expected", test_womens_data.items(), ids=test_womens_data.keys())
@pytest.mark.fast
def test_get_label(category: str, expected: str) -> None:
    label = womens_resolver_labels(category)
    assert label == expected


TEMPORAL_CASES = [
    ("test_prefix_bot_womens_1", test_womens_data, womens_resolver_labels),
]


@pytest.mark.dump
@pytest.mark.parametrize("name,data, callback", TEMPORAL_CASES)
def test_all_dump(name: str, data: dict[str, str], callback) -> None:
    expected, diff_result = one_dump_test(data, callback)

    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



# -*- coding: utf-8 -*-
"""
Single-file test implementation for the '{en_sport}' sports template resolver.

TODO: Use it in the code or remove it!

"""

import functools
import re
from typing import Dict

from ..helps import logger
from ..translations_formats import FormatData

TEMPLATES_TEAMS: Dict[str, str] = {
    "men's {en_sport} world cup": "كأس العالم للرجال في {sport_ar}",
    "women's {en_sport} world cup": "كأس العالم للسيدات في {sport_ar}",
    "{en_sport} world cup": "كأس العالم في {sport_ar}",
    "men's {en_sport} world championship": "بطولة العالم للرجال في {sport_ar}",
    "women's {en_sport} world championship": "بطولة العالم للسيدات في {sport_ar}",
    "{en_sport} world championship": "بطولة العالم في {sport_ar}",
    "men's {en_sport} asian championship": "بطولة آسيا للرجال في {sport_ar}",
    "women's {en_sport} asian championship": "بطولة آسيا للسيدات في {sport_ar}",
    "{en_sport} asian championship": "بطولة آسيا في {sport_ar}",
    "men's {en_sport} league": "دوري الرجال في {sport_ar}",
    "women's {en_sport} league": "دوري السيدات في {sport_ar}",
    "{en_sport} league": "الدوري في {sport_ar}",
    "men's {en_sport} cup": "كأس الرجال في {sport_ar}",
    "women's {en_sport} cup": "كأس السيدات في {sport_ar}",
    "{en_sport} cup": "الكأس في {sport_ar}",
    "u23 {en_sport} championship": "بطولة تحت 23 سنة في {sport_ar}",
    "u20 {en_sport} championship": "بطولة تحت 20 سنة في {sport_ar}",
    "u17 {en_sport} world cup": "كأس العالم تحت 17 سنة في {sport_ar}",
    "wheelchair {en_sport} world championship": "بطولة العالم للكراسي المتحركة في {sport_ar}",
    "wheelchair {en_sport}": "{sport_ar} على كراسي متحركة",
    "{en_sport} racing": "سباقات {sport_ar}",
    "men's national {en_sport} team": "منتخب {sport_ar} الوطني للرجال",
    "national women's {en_sport} team": "منتخب {sport_ar} الوطني للسيدات",
    "women's national {en_sport} team": "منتخب {sport_ar} الوطني للسيدات",
    "national {en_sport} team": "المنتخب الوطني في {sport_ar}",
}
# ---------- teamv_job.py ----------
SPORTS_EN_TO_AR: Dict[str, str] = {
    "association football": "كرة القدم",
    "football": "كرة القدم",
    "futsal": "كرة الصالات",
    "softball": "سوفتبول",
    "baseball": "بيسبول",
    "basketball": "كرة السلة",
    "volleyball": "كرة الطائرة",
    "handball": "كرة اليد",
    "rugby union": "اتحاد الرجبي",
    "rugby league": "رجبي ليغ",
    "hockey": "هوكي",
    "field hockey": "هوكي الحقول",
    "ice hockey": "هوكي الجليد",
    "cricket": "كريكت",
    "tennis": "تنس",
    "table tennis": "تنس الطاولة",
    "badminton": "بادمنتون",
    "wrestling": "مصارعة",
    "boxing": "ملاكمة",
    "kick boxing": "كيك بوكسينغ",
    "martial arts": "فنون قتالية",
    "aquatic sports": "رياضات مائية",
    "shooting": "رماية",
    "sport climbing": "تسلق",
    "motorsports": "رياضة محركات",
    "automobile racing": "سباق سيارات",
    "gaelic football": "كرة القدم الغيلية",
}

WHITESPACE_NORM = re.compile(r"\s+")


def _normalize(text: str) -> str:
    """Lowercase and collapse whitespace for consistent matching."""
    return WHITESPACE_NORM.sub(" ", text.lower()).strip().replace("–", "-")


def _expand_templates(templates: Dict[str, str]) -> Dict[str, str]:
    """Add relaxed variants that mirror the previous manual fallbacks."""

    expanded = dict(templates)

    for key, value in templates.items():
        relaxed_key = key.replace("men's", "mens").replace("women's", "womens")
        expanded.setdefault(relaxed_key, value)

        # tokens = ["{en_sport}" if token == "{en_sport}" else (token[:-1] if token.endswith("s") else token) for token in key.split(" ")]
        tokens = [
            token[:-1] if token == "championships" else token for token in key.split(" ") if token != "{en_sport}"
        ]
        alt_key = " ".join(tokens)
        expanded.setdefault(alt_key, value)

    return expanded


@functools.lru_cache(maxsize=1)
def _load_sports_bot() -> FormatData:
    """Create a shared FormatData instance for sports template resolution."""

    expanded_templates = _expand_templates(TEMPLATES_TEAMS)
    return FormatData(
        expanded_templates,
        SPORTS_EN_TO_AR,
        key_placeholder="{en_sport}",
        value_placeholder="{sport_ar}",
    )


@functools.lru_cache(maxsize=10000)
def resolve_team_label(title_en: str) -> str:
    """Resolve an Arabic team label for a sports title using FormatData."""
    logger.debug(f"<<yellow>> start resolve_by_labels: {title_en=}")
    bot = _load_sports_bot()
    normalized_title = _normalize(title_en)
    result = bot.search(normalized_title)
    logger.info_if_or_debug(f"<<yellow>> end resolve_by_labels: {title_en=}, {result=}", result)
    return result



#!/usr/bin/python3
"""
"""
import functools

from ...helps import logger
from ...translations import All_Nat, all_country_with_nat, countries_en_as_nationality_keys
from ...translations_formats import FormatDataDoubleV2
from ..nats_as_country_names import nats_keys_as_country_names

countries_en_keys = [x.get("en") for x in all_country_with_nat.values() if x.get("en")]

formatted_data = {
    "{en} border crossings": "معابر الحدود {the_female}",
    "{en} joint economic efforts": "الجهود الاقتصادية المشتركة {the_female}",
    "{en} conflict video games": "ألعاب فيديو الصراع {the_male}",
    "{en} conflict legal issues": "قضايا قانونية في الصراع {the_male}",
    "{en} conflict": "الصراع {the_male}",
    "{en} football rivalry": "التنافس {the_male} في كرة القدم",
    # "jewish persian": "فرس يهود",
    "{en}": "{males}",
    "{en} people": "{males}",
    # north american-jewish culture
    "{en} surnames": "ألقاب {female}",
    "{en} culture": "ثقافة {female}",
    "{en} war video games": "ألعاب فيديو الحرب {the_female}",
    "{en} war films": "أفلام الحرب {the_female}",
    "{en} families": "عائلات {female}",
    "{en} war": "الحرب {the_female}",
    "{en} wars": "الحروب {the_female}",
    "{en} television series": "مسلسلات تلفزيونية {female}",
    "{en} literature": "أدب {male}",
    "{en} history": "تاريخ {male}",
    "{en} cuisine": "مطبخ {male}",
    "{en} descent": "أصل {male}",
    "{en} diaspora": "شتات {male}",
    "{en} law": "قانون {male}",
    "{en} wine": "نبيذ {male}",
    "{en} traditions": "تراث {male}",
    "{en} folklore": "فلكور {male}",
    "{en} television": "تلفاز {male}",
    "{en} rock genres": "أنواع روك {male}",
    "{en} musical groups": "فرق موسيقية {female}",
    "{en} music": "موسيقى {female}",
    "{en} music genres": "أنواع موسيقى {female}",
    "{en} genres": "أنواع {female}",
    "{en} novels": "روايات {female}",
    "{en} architecture": "عمارة {female}",
    "{en} gangs": "عصابات {female}",
}

nats_data = {x: v for x, v in All_Nat.items()}
nats_data.update({x: v for x, v in nats_keys_as_country_names.items()})
nats_data.update(
    {
        "ireland": {
            "male": "أيرلندي",
            "males": "أيرلنديون",
            "female": "أيرلندية",
            "females": "أيرلنديات",
            "the_male": "الأيرلندي",
            "the_female": "الأيرلندية",
            "en": "ireland",
            "ar": "أيرلندا",
        }
    }
)


@functools.lru_cache(maxsize=1)
def double_bot() -> FormatDataDoubleV2:
    # Template data with both nationality and sport placeholders
    # "german jewish history": "تاريخ يهودي ألماني",

    # Create an instance of the FormatDataDoubleV2 class with the formatted data and data list
    _bot = FormatDataDoubleV2(
        formatted_data=formatted_data,
        data_list=nats_data,
        key_placeholder="{en}",
        splitter=r"[−–\- ]",
        sort_ar_labels=True,
    )

    return _bot


def fix_keys(category: str) -> str:
    """Fix known issues in category keys before searching.

    Args:
        category: The original category key.
    """
    # Fix specific known issues with category keys
    category = category.lower().replace("category:", "")
    category = category.replace("'", "")
    return category.strip()


@functools.lru_cache(maxsize=10000)
def resolve_by_nats_double_v2(category: str) -> str:
    category = fix_keys(category)
    logger.debug(f"<<yellow>> start resolve_by_nats_double_v2: {category=}")

    if category in countries_en_as_nationality_keys or category in countries_en_keys:
        logger.info(f"<<yellow>> skip resolve_by_nats_double_v2: {category=}, [result=]")
        return ""

    if category in nats_data:
        # NOTE: only nationality key should be handled by other resolvers
        logger.info(f"<<yellow>> skip resolve_by_nats_double_v2: one nationality key only {category=}, [result=]")
        return ""

    nat_bot = double_bot()
    result = nat_bot.search_all_category(category)
    logger.info_if_or_debug(f"<<yellow>> end resolve_by_nats_double_v2: {category=}, {result=}", result)
    return result


__all__ = [
    "resolve_by_nats_double_v2",
]



import functools

from ..helps import logger
from ..new.resolve_films_bots import get_films_key_tyty_new, get_films_key_tyty_new_and_time
from .countries_names_resolvers import resolve_countries_names_main
from .countries_names_with_sports import resolved_names_with_sports
from .jobs_resolvers import resolve_jobs_main
from .nationalities_resolvers import resolve_nationalities_main
from .relations_resolver import new_relations_resolvers
from .sports_resolvers import resolve_sports_main, sport_lab_nat
from .translations_resolvers_v3i import resolve_v3i_main


@functools.lru_cache(maxsize=None)
def new_resolvers_all(category: str) -> str:
    logger.debug(f">> new_resolvers_all: {category}")
    category_lab = (
        # resolve_jobs_main before sports, to avoid mis-resolving like:
        # incorrect:    "Category:American basketball coaches": "تصنيف:مدربو كرة سلة أمريكية"
        # correct:      "Category:American basketball coaches": "تصنيف:مدربو كرة سلة أمريكيون"
        resolve_jobs_main(category)
        or resolve_v3i_main(category)
        or resolve_sports_main(category)
        # NOTE: resolve_nationalities_main must be before resolve_countries_names_main to avoid conflicts like:
        # resolve_countries_names_main> [Italy political leader]:  "قادة إيطاليا السياسيون"
        # resolve_nationalities_main> [Italy political leader]:  "قادة سياسيون إيطاليون"
        or resolve_nationalities_main(category)
        or resolve_countries_names_main(category)
        or get_films_key_tyty_new_and_time(category)
        or get_films_key_tyty_new(category)
        or new_relations_resolvers(category)
        or sport_lab_nat.sport_lab_nat_load_new(category)
        or resolved_names_with_sports(category)
        or ""
    )
    logger.debug(f"<< new_resolvers_all: {category} => {category_lab}")
    return category_lab



#!/usr/bin/python3
"""
Film and TV Series Translation Mappings.

Builds translation mappings for film and television categories from English to Arabic,
handling gender-specific translations and nationality-based categories.
"""

from typing import Dict, Tuple

from ...helps import len_print
from ..utils.json_dir import open_json_file

# =============================================================================
# Constants
# =============================================================================

NAT_PLACEHOLDER = "{}"

# Keys that support debuts/endings variants
DEBUTS_ENDINGS_KEYS = ["television series", "television miniseries", "television films"]

# Fixed television/web series templates
SERIES_DEBUTS_ENDINGS = {
    "television-series debuts": "مسلسلات تلفزيونية {} بدأ عرضها في",
    "television-series endings": "مسلسلات تلفزيونية {} انتهت في",
    "web series-debuts": "مسلسلات ويب {} بدأ عرضها في",
    "web series debuts": "مسلسلات ويب {} بدأ عرضها في",
    "television series-debuts": "مسلسلات تلفزيونية {} بدأ عرضها في",
    "television series debuts": "مسلسلات تلفزيونية {} بدأ عرضها في",
    "television series endings": "مسلسلات تلفزيونية {} انتهت في",
}

# General television/media category base translations
TELEVISION_BASE_KEYS = {
    "video games": "ألعاب فيديو",
    "soap opera": "مسلسلات طويلة",
    "television characters": "شخصيات تلفزيونية",
    "television programs": "برامج تلفزيونية",
    "television programmes": "برامج تلفزيونية",
    "web series": "مسلسلات ويب",
    "television series": "مسلسلات تلفزيونية",
    "film series": "سلاسل أفلام",
    "television episodes": "حلقات تلفزيونية",
    "television news": "أخبار تلفزيونية",
    "comics": "قصص مصورة",
    "television films": "أفلام تلفزيونية",
    "miniseries": "مسلسلات قصيرة",
    "television miniseries": "مسلسلات قصيرة تلفزيونية",
}

# Extended television keys dictionary
TELEVISION_KEYS = {
    # "people": "أعلام",
    "albums": "ألبومات",
    "animation": "رسوم متحركة",
    "anime and manga": "أنمي ومانغا",
    "bodies": "هيئات",
    "championships": "بطولات",
    "clubs": "أندية",
    "clubs and teams": "أندية وفرق",
    "comic strips": "شرائط كومكس",
    "comics": "قصص مصورة",
    "competition": "منافسات",
    "competitions": "منافسات",
    "culture": "ثقافة",
    "equipment": "معدات",
    "executives": "مدراء",
    "films": "أفلام",
    "games": "ألعاب",
    "governing bodies": "هيئات تنظيم",
    "graphic novels": "روايات مصورة",
    "logos": "شعارات",
    "magazines": "مجلات",
    "manga": "مانغا",
    "media": "إعلام",
    "music": "موسيقى",
    "non-profit organizations": "منظمات غير ربحية",
    "non-profit publishers": "ناشرون غير ربحيون",
    "novellas": "روايات قصيرة",
    "novels": "روايات",
    "occupations": "مهن",
    "organizations": "منظمات",
    "short stories": "قصص قصيرة",
    "soap opera": "مسلسلات طويلة",
    "soundtracks": "موسيقى تصويرية",
    "tactics and skills": "مهارات",
    "teams": "فرق",
    "television commercials": "إعلانات تجارية تلفزيونية",
    "television episodes": "حلقات تلفزيونية",
    "television films": "أفلام تلفزيونية",
    "miniseries": "مسلسلات قصيرة",
    "television miniseries": "مسلسلات قصيرة تلفزيونية",
    "television news": "أخبار تلفزيونية",
    "television programmes": "برامج تلفزيونية",
    "television programming": "برمجة تلفزيونية",
    "television programs": "برامج تلفزيونية",
    "television schedules": "جداول تلفزيونية",
    "television series": "مسلسلات تلفزيونية",
    "film series": "سلاسل أفلام",
    "television shows": "عروض تلفزيونية",
    "terminology": "مصطلحات",
    "variants": "أشكال",
    "video games": "ألعاب فيديو",
    "web series": "مسلسلات ويب",
    "webcomic": "ويب كومكس",
    "webcomics": "ويب كومكس",
    "works": "أعمال",
}

# =============================================================================
# Helper Functions
# =============================================================================


def _build_gender_key_maps(
    films_key_o_multi: Dict[str, Dict[str, str]]
) -> Tuple[Dict[str, Dict[str, str]], Dict[str, str],]:  # films_key_both  # films_key_man
    """
    Build gender-aware film key mappings from JSON sources.

    Returns:
        - films_key_both: Lowercase key → {male, female}
        - films_key_man: Key → male label (with animated variants)
    """
    films_key_both = {}
    films_key_man = {}

    # Process films_key_o_multi
    for en_key, labels in films_key_o_multi.items():
        key_lower = en_key.lower()
        films_key_both[key_lower] = labels

    # Handle "animated" → "animation" aliasing
    if "animated" in films_key_both:
        films_key_both["animation"] = films_key_both["animated"]

    # Build gender-specific maps
    for en_key, labels in films_key_both.items():
        male_label = labels.get("male", "").strip()

        if male_label:
            films_key_man[en_key] = male_label
            # Add animated variant for male
            if "animated" not in en_key:
                films_key_man[f"animated {en_key}"] = f"{male_label} رسوم متحركة"

    return (
        films_key_both,
        films_key_man,
    )


def _extend_females_labels(
    films_keys_male_female: Dict[str, Dict[str, str]],
) -> Dict[str, str]:
    """
    Extract female labels from the male/female dictionary with animation aliasing.

    Processes the input dictionary and returns a mapping of original keys to their
    female labels. Includes special handling to alias "animated" to "animation".

    Args:
        films_keys_male_female: Dictionary mapping English keys to gender label pairs

     Returns:
        Dictionary mapping original keys to female labels
    """
    data = {}

    # Process films_keys_male_female (with animation aliasing)
    male_female_copy = dict(films_keys_male_female)
    if "animated" in male_female_copy:
        male_female_copy["animation"] = male_female_copy["animated"]

    for en_key, labels in male_female_copy.items():
        female_label = labels.get("female", "").strip()
        if female_label:
            data[en_key] = female_label

    return data


def _build_series_and_nat_keys(
    female_keys: Dict[str, str],
) -> Tuple[Dict[str, str], Dict[str, str]]:
    """
    Build nationality-aware and series-based translation mappings.

    Returns:
        - films_key_for_nat: With nationality placeholder {}
        - films_mslslat_tab: Without nationality placeholder
    """
    _mslslat_tab = {}
    _key_for_nat = {}

    # Add fixed templates
    _key_for_nat.update(SERIES_DEBUTS_ENDINGS)

    # Add remakes mapping
    _key_for_nat["remakes of {} films"] = f"أفلام {NAT_PLACEHOLDER} معاد إنتاجها"

    # Build base series keys
    for tt, tt_lab in TELEVISION_BASE_KEYS.items():
        _key_for_nat[tt] = f"{tt_lab} {NAT_PLACEHOLDER}"
        _mslslat_tab[tt] = tt_lab

        # Debuts, endings, revived variants
        for suffix, arabic_suffix in [
            ("debuts", "بدأ عرضها في"),
            ("endings", "انتهت في"),
            ("revived after cancellation", "أعيدت بعد إلغائها"),
        ]:
            key_with_suffix = f"{tt} {suffix}"
            _key_for_nat[key_with_suffix] = f"{tt_lab} {NAT_PLACEHOLDER} {arabic_suffix}"
            _mslslat_tab[key_with_suffix] = f"{tt_lab} {arabic_suffix}"

        # Dashed variants for specific keys
        if tt.lower() in DEBUTS_ENDINGS_KEYS:
            for suffix, arabic_suffix in [("debuts", "بدأ عرضها في"), ("endings", "انتهت في")]:
                dashed_key = f"{tt}-{suffix}"
                _key_for_nat[dashed_key] = f"{tt_lab} {NAT_PLACEHOLDER} {arabic_suffix}"
                _mslslat_tab[dashed_key] = f"{tt_lab} {arabic_suffix}"

    # Build combinations of female film keys with series keys
    for ke, ke_lab in female_keys.items():
        for tt, tt_lab in TELEVISION_BASE_KEYS.items():
            key_base = f"{ke} {tt}"

            # Base combination
            _key_for_nat[key_base] = f"{tt_lab} {ke_lab} {NAT_PLACEHOLDER}"
            _mslslat_tab[key_base] = f"{tt_lab} {ke_lab}"

            # Debuts, endings, revived variants
            for suffix, arabic_suffix in [
                ("debuts", "بدأ عرضها في"),
                ("endings", "انتهت في"),
                ("revived after cancellation", "أعيدت بعد إلغائها"),
            ]:
                combo_key = f"{key_base} {suffix}"

                if suffix == "revived after cancellation":
                    _key_for_nat[combo_key] = f"{tt_lab} {ke_lab} {NAT_PLACEHOLDER} {arabic_suffix}"
                    _mslslat_tab[combo_key] = f"{tt_lab} {ke_lab} {arabic_suffix}"
                else:
                    _key_for_nat[combo_key] = f"{tt_lab} {ke_lab} {NAT_PLACEHOLDER} {arabic_suffix}"
                    _mslslat_tab[combo_key] = f"{tt_lab} {ke_lab} {arabic_suffix}"

            # Dashed variants
            if tt.lower() in DEBUTS_ENDINGS_KEYS:
                for suffix, arabic_suffix in [("debuts", "بدأ عرضها في"), ("endings", "انتهت في")]:
                    dashed_key = f"{key_base}-{suffix}"
                    _key_for_nat[dashed_key] = f"{tt_lab} {ke_lab} {NAT_PLACEHOLDER} {arabic_suffix}"
                    _mslslat_tab[dashed_key] = f"{tt_lab} {ke_lab} {arabic_suffix}"

    return _key_for_nat, _mslslat_tab


def _build_television_cao(
    female_keys: Dict[str, str],
) -> Tuple[Dict[str, str], int]:
    """
    Build CAO (Characters, Albums, Organizations, etc.) mappings.

    Returns:
        - films_key_cao: CAO translation mapping
        - count: Number of genre-TV combinations created
    """
    films_key_cao = {}
    count = 0

    # Base TV keys with common suffixes
    for ff, label in TELEVISION_KEYS.items():
        films_key_cao[ff] = label
        for suffix, arabic_suffix in [
            ("characters", "شخصيات"),
            ("title cards", "بطاقات عنوان"),
            ("video covers", "أغلفة فيديو"),
            ("posters", "ملصقات"),
            ("images", "صور"),
        ]:
            films_key_cao[f"{ff} {suffix}"] = f"{arabic_suffix} {label}"

    # Genre-based categories
    genre_categories = [
        ("anime and manga", "أنمي ومانغا"),
        ("compilation albums", "ألبومات تجميعية"),
        ("folk albums", "ألبومات فلكلورية"),
        ("classical albums", "ألبومات كلاسيكية"),
        ("comedy albums", "ألبومات كوميدية"),
        ("mixtape albums", "ألبومات ميكستايب"),
        ("soundtracks", "موسيقى تصويرية"),
        ("terminology", "مصطلحات"),
        ("television series", "مسلسلات تلفزيونية"),
        ("television episodes", "حلقات تلفزيونية"),
        ("television programs", "برامج تلفزيونية"),
        ("television programmes", "برامج تلفزيونية"),
        ("groups", "مجموعات"),
        ("novellas", "روايات قصيرة"),
        ("novels", "روايات"),
        ("films", "أفلام"),
    ]

    for ke, ke_lab in female_keys.items():
        # Special cases
        films_key_cao[f"children's {ke}"] = f"أطفال {ke_lab}"
        films_key_cao[f"{ke} film remakes"] = f"أفلام {ke_lab} معاد إنتاجها"

        # Standard categories
        for suffix, arabic_base in genre_categories:
            films_key_cao[f"{ke} {suffix}"] = f"{arabic_base} {ke_lab}"

        # Combinations with all TV keys
        for fao, base_label in TELEVISION_KEYS.items():
            count += 1
            films_key_cao[f"{ke} {fao}"] = f"{base_label} {ke_lab}"

    return films_key_cao, count


def _build_female_combo_keys(
    filmskeys_male_female: Dict[str, Dict[str, str]],
) -> Dict[str, str]:
    """Build all pairwise combinations of female genre labels."""
    result = {}

    # Extract female labels
    base_female = {x: v["female"] for x, v in filmskeys_male_female.items() if v.get("female", "").strip()}

    # Generate combinations
    for en, tab in filmskeys_male_female.items():
        tab_female = tab.get("female", "").strip()
        if not tab_female:
            continue

        for en2, tab2_female in base_female.items():
            if en == en2:
                continue
            new_key = f"{en} {en2}".lower()
            if tab2_female:
                result[new_key] = f"{tab_female} {tab2_female}"

    return result


def build_gender_specific_film_maps(
    Films_keys_male_female: Dict[str, Dict[str, str]],
    Films_key_O_multi: Dict[str, Dict[str, str]],
    films_key_both: Dict[str, Dict[str, str]],
) -> tuple[dict, dict]:
    """
    Build gender-aware film key mappings.

    Returns:
        - Films_key_333: key → {male, female}
        - film_keys_for_female: Key → female label
    """
    Films_key_333: Dict[str, str] = {}
    # Build gender-specific maps

    film_keys_for_female: Dict[str, str] = {
        x: v.get("female", "").strip() for x, v in films_key_both.items() if v.get("female", "").strip()
    }

    female_extended_labels = _extend_females_labels(Films_keys_male_female)
    Films_key_333.update(female_extended_labels)
    film_keys_for_female.update(female_extended_labels)

    # Extend Films_key_333 with female labels from Films_key_O_multi
    for cd, ff in Films_key_O_multi.items():
        female_label = ff.get("female", "").strip()
        if female_label:  # and cd not in Films_key_333:
            Films_key_333[cd] = female_label

    return Films_key_333, film_keys_for_female


# =============================================================================
# Module Initialization
# =============================================================================

# Load JSON resources
Films_key_For_nat = open_json_file("media/Films_key_For_nat.json") or {}
_Films_key_O_multi = open_json_file("media/Films_key_O_multi.json") or {}

Films_keys_male_female = open_json_file("media/Films_keys_male_female.json") or {}
Films_keys_male_female["sports"] = {"male": "رياضي", "female": "رياضية"}
# Films_keys_male_female["superhero"] = {"male": "خارق", "female": "أبطال خارقين"}

# Filter to only entries with both male and female
Films_key_O_multi = {
    x: v for x, v in _Films_key_O_multi.items() if v.get("male", "").strip() and v.get("female", "").strip()
}

# Build gender-aware mappings
(
    Films_key_both,
    Films_key_man,
) = _build_gender_key_maps(Films_key_O_multi)

film_keys_for_male: Dict[str, str] = {
    x: v.get("male", "").strip() for x, v in Films_key_both.items() if v.get("male", "").strip()
}

Films_key_333, film_keys_for_female = build_gender_specific_film_maps(
    Films_keys_male_female, Films_key_O_multi, Films_key_both
)


# Build series and nationality keys
# films_key_for_nat_extended_org, films_mslslat_tab_base_org = _build_series_and_nat_keys(film_keys_for_female)

films_mslslat_tab_base = open_json_file("films_mslslat_tab_found.json")

# Films_key_For_nat_extended = open_json_file("Films_key_For_nat_extended_found.json")
# NOTE: "Films_key_For_nat_extended_found.json" and "films_mslslat_tab_found.json" looks the same exept Films_key_For_nat_extended_found has placeholder {} in values

Films_key_For_nat_extended = {x: f"{v} {{}}" for x, v in films_mslslat_tab_base.items()}

films_mslslat_tab = dict(films_mslslat_tab_base)

films_mslslat_tab.update(
    {
        "science fiction film series-endings": "سلاسل أفلام خيال علمي انتهت في",
        "science fiction film series debuts": "سلاسل أفلام خيال علمي بدأ عرضها في",
        "television series revived after cancellation": "مسلسلات تلفزيونية أعيدت بعد إلغائها",
        "comics endings": "قصص مصورة انتهت في",
        "television series endings": "مسلسلات تلفزيونية انتهت في",
        "animated television series endings": "مسلسلات تلفزيونية رسوم متحركة انتهت في",
        "web series endings": "مسلسلات ويب انتهت في",
        "web series debuts": "مسلسلات ويب بدأ عرضها في",
        "anime television series debuts": "مسلسلات تلفزيونية أنمي بدأ عرضها في",
        "comics debuts": "قصص مصورة بدأ عرضها في",
        "animated television series debuts": "مسلسلات تلفزيونية رسوم متحركة بدأ عرضها في",
        "television series debuts": "مسلسلات تلفزيونية بدأ عرضها في",
        "supernatural television series": "مسلسلات تلفزيونية خارقة للطبيعة",
        "supernatural comics": "قصص مصورة خارقة للطبيعة",
        "adult animated supernatural television series": "مسلسلات تلفزيونية رسوم متحركة خارقة للطبيعة للكبار",
        "superhero television characters": "شخصيات تلفزيونية أبطال خارقين",
        "superhero television series": "مسلسلات تلفزيونية أبطال خارقين",
        "superhero film series": "سلاسل أفلام أبطال خارقين",
        "superhero television episodes": "حلقات تلفزيونية أبطال خارقين",
        "superhero video games": "ألعاب فيديو أبطال خارقين",
        "superhero web series": "مسلسلات ويب أبطال خارقين",
        "superhero comics": "قصص مصورة أبطال خارقين",
        "superhero television films": "أفلام تلفزيونية أبطال خارقين",
    }
)

films_mslslat_tab.update(
    {x.replace(" endings", "-endings"): y for x, y in films_mslslat_tab.items() if " endings" in x}
)

Films_key_For_nat.update(
    {
        "drama films": "أفلام درامية {}",
        "legal drama films": "أفلام قانونية درامية {}",
        # "yemeni musical drama films" : "تصنيف:أفلام موسيقية درامية يمنية",
        "musical drama films": "أفلام موسيقية درامية {}",
        "political drama films": "أفلام سياسية درامية {}",
        "romantic drama films": "أفلام رومانسية درامية {}",
        "sports drama films": "أفلام رياضية درامية {}",
        "comedy drama films": "أفلام كوميدية درامية {}",
        "war drama films": "أفلام حربية درامية {}",
        "action drama films": "أفلام حركة درامية {}",
        "adventure drama films": "أفلام مغامرات درامية {}",
        "animated drama films": "أفلام رسوم متحركة درامية {}",
        "children's drama films": "أفلام أطفال درامية {}",
        "crime drama films": "أفلام جريمة درامية {}",
        "erotic drama films": "أفلام إغرائية درامية {}",
        "fantasy drama films": "أفلام فانتازيا درامية {}",
        "horror drama films": "أفلام رعب درامية {}",
    }
)
Films_key_For_nat.update(Films_key_For_nat_extended)

Films_key_For_nat.update(
    {
        "science fiction film series endings": "سلاسل أفلام خيال علمي {} انتهت في",
        "science fiction film series debuts": "سلاسل أفلام خيال علمي {} بدأ عرضها في",
        "television series revived after cancellation": "مسلسلات تلفزيونية {} أعيدت بعد إلغائها",
        "web series endings": "مسلسلات ويب {} انتهت في",
        "animated television series endings": "مسلسلات تلفزيونية رسوم متحركة {} انتهت في",
        "comics endings": "قصص مصورة {} انتهت في",
        "television series endings": "مسلسلات تلفزيونية {} انتهت في",
        "television series debuts": "مسلسلات تلفزيونية {} بدأ عرضها في",
        "comics debuts": "قصص مصورة {} بدأ عرضها في",
        "animated television series debuts": "مسلسلات تلفزيونية رسوم متحركة {} بدأ عرضها في",
        "web series debuts": "مسلسلات ويب {} بدأ عرضها في",
        "anime television series debuts": "مسلسلات تلفزيونية أنمي {} بدأ عرضها في",
        "supernatural television series": "مسلسلات تلفزيونية خارقة للطبيعة {}",
        "supernatural comics": "قصص مصورة خارقة للطبيعة {}",
        "adult animated supernatural television series": "مسلسلات تلفزيونية رسوم متحركة خارقة للطبيعة للكبار {}",
        "superhero film series": "سلاسل أفلام أبطال خارقين {}",
        "superhero television episodes": "حلقات تلفزيونية أبطال خارقين {}",
        "superhero video games": "ألعاب فيديو أبطال خارقين {}",
        "superhero web series": "مسلسلات ويب أبطال خارقين {}",
        "superhero television films": "أفلام تلفزيونية أبطال خارقين {}",
        "superhero comics": "قصص مصورة أبطال خارقين {}",
        "superhero television characters": "شخصيات تلفزيونية أبطال خارقين {}",
        "superhero television series": "مسلسلات تلفزيونية أبطال خارقين {}",
    }
)

# Build television CAO mappings
Films_key_CAO, ss_Films_key_CAO = _build_television_cao(film_keys_for_female)

# Build female combination keys
# Films_keys_both_new_female = _build_female_combo_keys(Films_keys_male_female)
Films_keys_both_new_female = open_json_file("Films_keys_both_new_female_found.json")

# Legacy aliases
film_key_women_2 = TELEVISION_BASE_KEYS
television_keys = TELEVISION_KEYS

# Summary output
len_print.data_len(
    "films_mslslat.py",
    {
        "Films_key_For_nat_extended": Films_key_For_nat_extended,
        "television_keys": television_keys,
        "Films_key_For_nat": Films_key_For_nat,
        "films_mslslat_tab": films_mslslat_tab,
        "ss_Films_key_CAO": ss_Films_key_CAO,
        "Films_key_333": Films_key_333,
        "Films_key_CAO": Films_key_CAO,
        "Films_keys_both_new_female": Films_keys_both_new_female,
        "film_keys_for_female": film_keys_for_female,
        "film_keys_for_male": film_keys_for_male,
        "Films_key_man": Films_key_man,
        "film_key_women_2": film_key_women_2,
        # "films_key_for_nat_extended_org": films_key_for_nat_extended_org,
        # "films_mslslat_tab_base_org": films_mslslat_tab_base_org,
    },
)


# =============================================================================
# Public API
# =============================================================================

__all__ = [
    "television_keys",
    "films_mslslat_tab",
    "film_keys_for_female",
    "film_keys_for_male",
    "Films_key_333",
    "Films_key_CAO",
    "Films_key_For_nat",
    "Films_key_man",
    "Films_keys_both_new_female",
    "film_key_women_2",
]
